#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")

parser.add_argument("--burn_in_factor", "-b", 
  default='4',
  help='Number of burn-in steps = BURN_IN_FACTOR x autocorrelation length')

args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')
import numpy as np
import corner
import matplotlib.pyplot as plt
from astropy.table import Table, Column
from emcee.autocorr import function as autocorr_function

#------------------------------------------------------------------------------

# Handy class for printing floats...
class prettyfloat(float):
  def __repr__(self):
    return "%0.6f" % self

def _autocorrelation_length(v):
  """
  Estimate the autocorrelation length of a time series

  Finds the point where the autocorrelation function drops to e^-1
  """
  x = np.arange(len(v))
  acf = autocorr_function(v)
  return np.argmax(x[acf > np.exp(-1)])

#------------------------------------------------------------------------------
# Generate plots


chain = Table.read('chain.csv')
chain = Table(chain, masked=True)

nwalkers = 1+np.max(chain['walker'])
nsteps = 1+np.max(chain['step'])
print('Read chain.csv with {:d} walkers of {:d} steps'
    .format(nwalkers,nsteps))

step = chain['step']
walker = chain['walker']
chain.remove_column('step')
chain.remove_column('walker')
chain.remove_column('loglike')
lrat = chain['sb2']*(chain['r_2']/chain['r_1'])**2
chain.add_column(Column(lrat,name='lrat'))

ndim = len(chain.colnames)
  
acl = np.zeros([ndim,nwalkers],dtype=int)
print('\nCalculating autocorrelation length of chains for each walker.')
print('  For each parameter, results are quoted as median and range.')   
for icol,colname in enumerate(chain.colnames):
  p =  (chain[colname]).reshape([nwalkers,nsteps])
  for ii in range(0,nwalkers):
    acl[icol,ii] = _autocorrelation_length(p[ii,:])
  print(' {:s} : {} ({} - {})'.format(colname,
        np.median(acl[icol,:]),np.min(acl[icol,:]),np.max(acl[icol,:])))

burnin = np.zeros(nwalkers,dtype=int)
for ii in range(0,nwalkers):
  burnin[ii] = np.int(args.burn_in_factor)*np.max(acl[:,ii])
mask = (step == 0)
for ii in range(0,nwalkers):
  mask[walker == ii] = step[walker == ii] < burnin[ii]
nok = len(chain) - len(mask.nonzero()[0])

print ('Calculating results for chains with total of {:d} steps'.format(nok))

print("\n Median and std. dev. of parameter values.")  
for icol,colname in enumerate(chain.colnames):
  p =  np.extract(mask == False,chain[colname])
  if colname == 'T_0' :
    print('{:s} = {:14.6f} +/- {:8.6f}'.format(colname, 
      np.median(p),  np.std(p)))
  elif colname == 'P' :
    print('{:s} = {:12.8f} +/- {:10.8f}'.format(colname, 
      np.median(p),  np.std(p)))
  else:
    print('{:s} = {:10.6g} +/- {:10.6g}'.format(colname, 
      np.median(p),  np.std(p)))

colnames_plot = ['r_1','r_2','incl','sb2','lrat']
labels_plot = ['$r_1$','$r_2$','$i [^{\circ}]$','$s$','$l_2/l_1$']
nplot = len(colnames_plot)
medians = np.zeros(nplot)
samples = np.zeros([nok,nplot])
for ii,colname in enumerate(colnames_plot) :
  p =  np.extract(mask == False,chain[colname])
  samples[:,ii] = p
  medians[ii] = np.median(p)

# Results from Trevor et al., arXiv:1602.01901v1
previous = [0.1450, 0.1262, 78.21, 0.4859, 0.355]
# Plot parameter distributions
fig_p = corner.corner(samples, labels=labels_plot, truths=previous)
if args.eps:
  fig_p.savefig("corner.eps")
else:
  plt.show()






#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import ellc
import matplotlib.pyplot as plt


lc_dat = np.loadtxt("NightfallCurve.dat")

phase = lc_dat[:,1]
lc_q = 10**(-0.4*lc_dat[:,6])  # 
lc_q = lc_q /np.min(lc_q)
rv_q = lc_dat[:,15]  #   RV(Secondary)

period = 0.103
t_zero = 0.5*period
time = period*phase
sbratio = 0.0001
r_1 = 0.023765
r_2 = 0.169751
incl = 29.3
q = 0.234
ld_1 = 'quad'
ldc_1 = [0.028,  0.264]
ld_2 = 'quad'
ldc_2 = [0.272, 0.567]
shape = 'roche'
heat = [0.6,3.5,0.0]
dummy,rv_0 = ellc.rv(time,t_zero=t_zero, q=q, heat_2 = heat, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, a=0.737,
    period=period, shape_1=shape,shape_2=shape,flux_weighted=False) 

lc_1 = ellc.lc(time,t_zero=0.0, q=1/q, heat_1 = heat, 
    radius_1=r_2, radius_2=r_1,incl=incl,sbratio=1./sbratio,  a=0.737,
    period=period, ld_1=ld_2, ldc_1=ldc_2, ld_2=ld_1, ldc_2=ldc_1, 
    shape_1=shape,shape_2=shape) 
lc_1 = lc_1 /np.min(lc_1)
rv_1,dummy = ellc.rv(time,t_zero=0.0, q=1/q, heat_1 = heat, 
    radius_1=r_2, radius_2=r_1,incl=incl,sbratio=1./sbratio,  a=0.737,
    period=period, ld_1=ld_2, ldc_1=ldc_2, ld_2=ld_1, ldc_2=ldc_1, 
    shape_1=shape,shape_2=shape) 

# Test here is the same results are found if role of star 1 and star 2 are
# swapped (get test for coordinate transforms in stellar.f90/bright()
lc_2 = ellc.lc(time,t_zero=t_zero, q=q, heat_2 = heat, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  a=0.737,
    period=period, ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, 
    shape_1=shape,shape_2=shape) 
lc_2 = lc_2 /np.min(lc_2)
dummy,rv_2 = ellc.rv(time,t_zero=t_zero, q=q, heat_2 = heat, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  a=0.737,
    period=period, ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, 
    shape_1=shape,shape_2=shape) 

fig=plt.figure(1,figsize=(8,8))

plt.subplot(211)
line_q,=plt.plot(phase,lc_q,color='k',label='Nightfall')
line_1,=plt.plot(phase,lc_1,color='g',linestyle='--',label='ellc 1')
line_2,=plt.plot(phase,lc_2,color='r',linestyle='-.',label='ellc 2')
plt.xlim([-0.25,0.75])
plt.xlabel("Phase")
plt.ylabel("Flux")
plt.legend(handles=[line_1,line_2,line_q],loc='upper right')

plt.subplot(212)
line_q,=plt.plot(phase,rv_q,color='k',label='Nightfall')
line_0,=plt.plot(phase,rv_0,color='b',linestyle=':',label='c-of-m')
line_1,=plt.plot(phase,rv_1,color='g',linestyle='--',label='ellc 1')
line_2,=plt.plot(phase,rv_2,color='r',linestyle='-.',label='ellc 2')
plt.xlim([-0.25,0.75])
plt.ylim([-200,200])
plt.xlabel("Phase")
plt.ylabel("RV [km/s]")
plt.legend(handles=[line_0,line_1,line_2,line_q],loc='lower right')

plt.tight_layout()

if args.eps:
  fig.savefig("GD448_rv.eps")
else:
  plt.show()



#!/usr/bin/env python
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import ellc
import batman
import numpy as np
import matplotlib.pyplot as plt

params = batman.TransitParams()
params.t0 = 0.                      #time of inferior conjunction
params.per = 1.                     #orbital period
params.rp = 0.1                     #planet radius (in units of stellar radii)
params.a = 10.                      #semi-major axis (in units of stellar radii)
params.inc = 90.                    #orbital inclination (in degrees)
params.ecc = 0.1                    #eccentricity
params.w = 60.                      #longitude of periastron (in degrees)
params.u = [0.1, 0.3]               #limb darkening coefficients
params.limb_dark = "quadratic"      #limb darkening model

t = np.linspace(-0.05, 0.05, 1000)

m = batman.TransitModel(params, t, max_err=0.01)   #initializes model
flux_batman = m.light_curve(params)  #calculates light curve

r_1 = 1./params.a 
r_2 = r_1*params.rp
incl = params.inc
ldc_1 = params.u 
ld_1 = 'quad'
sbratio = 0 
f_s = np.sqrt(params.ecc)*np.sin(params.w*np.pi/180.)
f_c = np.sqrt(params.ecc)*np.cos(params.w*np.pi/180.)
flux_ellc_d = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='sphere',
    grid_1='default',grid_2='default', f_s=f_s, f_c=f_c)
flux_ellc_f = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='sphere',
    grid_1='fine',grid_2='fine', f_s=f_s, f_c=f_c)
flux_ellc_s = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='sphere',
    grid_1='sparse',grid_2='sparse', f_s=f_s, f_c=f_c)

fig=plt.figure(1,figsize=(8,12))
plt.subplot(411)
plt.xlim([-0.05,0.05])
plt.ylim([0.988,1.002])
plt.plot(t, flux_batman,color='k')
plt.ylabel("Flux",fontsize=16)

plt.subplot(412)
plt.ylim([-20,20])
plt.xlim([-0.05,0.05])
line_d, = plt.plot(t, 1e6*(flux_ellc_d-flux_batman),label='default')
line_f, = plt.plot(t, 1e6*(flux_ellc_f-flux_batman),'--',label='fine')
line_s, = plt.plot(t, 1e6*(flux_ellc_s-flux_batman),':',label='sparse')
plt.ylabel("Flux difference [ppm]",fontsize=16)
plt.legend(handles=[line_d,line_f,line_s],loc='upper left')

# Polytrope and Roche potential
q = 0.001
flux_ellc_r = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    q=q,ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='roche', 
    f_s=f_s, f_c=f_c)
flux_ellc_p = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    q=q,ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='poly1p5', 
    f_s=f_s, f_c=f_c)
plt.subplot(413)
plt.ylim([-20,20])
plt.xlim([-0.05,0.05])
line_p, = plt.plot(t, 1e6*(flux_ellc_p-flux_batman),label='Polytrope')
line_r, = plt.plot(t, 1e6*(flux_ellc_r-flux_batman),'--',label='Roche')
plt.legend(handles=[line_p,line_r],loc='upper left')
plt.ylabel("Flux difference [ppm]",fontsize=16)

# Compare for i<90
del params
params = batman.TransitParams()
params.t0 = 0.                      #time of inferior conjunction
params.per = 1.                     #orbital period
params.rp = 0.1                     #planet radius (in units of stellar radii)
params.a = 10.                      #semi-major axis (in units of stellar radii)
params.inc = 87.                    #orbital inclination (in degrees)
params.ecc = 0.1                    #eccentricity
params.w = 60.                      #longitude of periastron (in degrees)
params.u = [0.1, 0.3]               #limb darkening coefficients
params.limb_dark = "quadratic"      #limb darkening model

del m
m = batman.TransitModel(params, t, max_err=0.01)   #initializes model
flux_batman = m.light_curve(params)  #calculates light curve
r_1 = 1./params.a 
r_2 = r_1*params.rp
incl = params.inc
ldc_1 = params.u 
ld_1 = 'quad'
sbratio = 0 
f_s = np.sqrt(params.ecc)*np.sin(params.w*np.pi/180.)
f_c = np.sqrt(params.ecc)*np.cos(params.w*np.pi/180.)
flux_ellc = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='sphere',
    grid_1='default',grid_2='default', f_s=f_s, f_c=f_c)
plt.subplot(414)
plt.ylim([-80,80])
plt.xlim([-0.05,0.05])
line, = plt.plot(t, 1e6*(flux_ellc-flux_batman),label='$i=87^{\circ}$')
plt.legend(handles=[line],loc='upper left')
plt.ylabel("Flux difference [ppm]",fontsize=16)
plt.xlabel("Time from central transit",fontsize=16)


plt.tight_layout()
if args.eps:
  fig.savefig("batman.eps")
else:
  plt.show()



#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')
import numpy as np
import ellc
import emcee
import scipy.optimize as op
import corner
import matplotlib.pyplot as plt


#------------------------------------------------------------------------------

# Handy class for printing floats...
class prettyfloat(float):
  def __repr__(self):
    return "%0.6f" % self

#------------------------------------------------------------------------------

#  Likelihood function
def lnlike(par,
           lc_wasp,lc_nites,lc_oed,lc_byu,lc_kpno,radvel,
           period, t_zero, q, ldc_white, ldc_iband, ldc_jband, 
           n_int_nites, n_int_kpno):

  r_1, r_2, incl, f_s, f_c, sbratio_jband, K_1 = par

# For the WASP data, only need to calculate the light curve in the region of
# the transit. Phase here puts transit at phase 0.5 (for convenince).
  ph_wasp = (1.5 + (((lc_wasp['HJD']-t_zero)/period) % 1)) % 1
  m = np.zeros_like(lc_wasp['HJD'])
  try:
    m_tr = -2.5*np.log10(ellc.lc((lc_wasp['HJD'])[abs(ph_wasp-0.5) < 0.02], 
      radius_1=r_1, radius_2=r_2, incl=incl, sbratio=0,f_s=f_s, f_c=f_c,
      ld_1='quad', ldc_1 = ldc_white, period=period, t_zero=t_zero ,
      grid_1='sparse',grid_2='sparse'))
    m[abs(ph_wasp-0.5) < 0.02] = m_tr
    res = lc_wasp['dmag']-m
    wt = 1./lc_wasp['e_dmag']**2
    zp = np.sum(res*wt)/np.sum(wt)
    chisq_wasp = np.sum((res-zp)**2*wt)
  except:
    chisq_wasp = 1e20
  
  try:
    m = -2.5*np.log10(ellc.lc(lc_nites['HJD'], radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_white,
      period=period, t_zero=t_zero, n_int=n_int_nites ,
      grid_1='sparse',grid_2='sparse') )
    res = lc_nites['dmag']-m
    wt = 1./lc_nites['e_dmag']**2
    zp = np.sum(res*wt)/np.sum(wt)
    chisq_nites = np.sum((res-zp)**2*wt)
  except:
    chisq_nites = 1e20
  
  try:
    m = -2.5*np.log10(ellc.lc(lc_oed['HJD'], radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_iband,
      period=period, t_zero=t_zero , grid_1='sparse',grid_2='sparse'))
    res = lc_oed['dmag']-m
    wt = 1./lc_oed['e_dmag']**2
    zp = np.sum(res*wt)/np.sum(wt)
    chisq_oed = np.sum((res-zp)**2*wt)
  except:
    chisq_oed = 1e20
  
  try:
    m = -2.5*np.log10(ellc.lc(lc_byu['HJD'], radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_iband,
      period=period, t_zero=t_zero, grid_1='sparse',grid_2='sparse' ))
    res = lc_byu['dmag']-m
    wt = 1./lc_byu['e_dmag']**2
    zp = np.sum(res*wt)/np.sum(wt)
    chisq_byu = np.sum((res-zp)**2*wt)
  except:
    chisq_byu = 1e20
    
  # Calculate semi-major axis
  ecc = f_s**2 + f_c**2
  a_1 = 0.019771142 * K_1 * period * np.sqrt(1 - ecc**2)/np.sin(incl*np.pi/180)
  a = (1+1/q)*a_1

  # kpno secondary eclipse - include semi-major axis for light time correction
  try:
    m = -2.5*np.log10(ellc.lc(lc_kpno['HJD'], radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=sbratio_jband,f_s=f_s, f_c=f_c, ld_2='quad', a=a, 
      ldc_2 = ldc_jband, period=period, t_zero=t_zero, n_int=n_int_kpno ,
      grid_1='sparse',grid_2='sparse'))
    res = lc_kpno['dmag']-m
    wt = 1./lc_kpno['e_dmag']**2
    zp = np.sum(res*wt)/np.sum(wt)
    chisq_kpno = np.sum((res-zp)**2*wt)
  except:
    chisq_kpno = 1e20

  # Radial velocity
  try:
    rv1,rv2 = ellc.rv(radvel['HJD'], radius_1=r_1, radius_2=r_2, incl=incl, 
                 q=q,sbratio=0,f_s=f_s, f_c=f_c, a=a, period=period, 
                 t_zero=t_zero, flux_weighted=False,
                 grid_1='very_sparse',grid_2='very_sparse')
    res = radvel['RV']-rv1
    wt = 1./radvel['e_RV']**2
    zp = np.sum(res*wt)/np.sum(wt)
    chisq_rv = np.sum((res-zp)**2*wt)
  except:
    chisq_rv = 1e20

  chisq = (chisq_wasp + chisq_nites + chisq_byu + chisq_oed + chisq_kpno + 
           chisq_rv)
  print(list(map(prettyfloat,[r_1, r_2, incl, f_s, f_c, sbratio_jband,
    K_1,chisq])))
  return -0.5*chisq
  
#------------------------------------------------------------------------------
# Generate plots

# Load data from ftp://cdsarc.u-strasbg.fr/pub/cats/J/A%2BA/572/A50/
colnames = (str('HJD'),str('dmag'),str('e_dmag'),str('Band'))
lc_wasp  = np.loadtxt('table5.dat',
                      dtype={'names': colnames,
                             'formats': ('f8','f4','f4','S13')})
lc_nites = np.loadtxt('table6.dat',
                      dtype={'names': colnames,
                             'formats': ('f8','f4','f4','S13')})
lc_oed   = np.loadtxt('table7.dat',
                      dtype={'names': colnames,
                             'formats': ('f8','f4','f4','S13')})
lc_byu   = np.loadtxt('table8.dat',
                      dtype={'names': colnames,
                             'formats': ('f8','f4','f4','S13')})
lc_kpno  = np.loadtxt('table9.dat',
                       dtype={'names': colnames,
                              'formats': ('f8','f4','f4','S13')})
colnames = (str('HJD'),str('RV'),str('e_RV'),str('Inst'))
radvel   = np.loadtxt('table10.dat',
                       dtype={'names': colnames,
                              'formats': ('f8','f4','f4','S8')})

period = 14.2769001
t_zero = 2456023.26988


# Interpolate 9/10 points for nites data.
n_int_nites = np.zeros_like(lc_nites['HJD'])
n_int_nites[::10] = 1
# Make sure first/last points after the large break and the last point in the 
# data are calculated and not interpolated
igap = np.argmax((lc_nites['HJD'])[lc_nites['HJD'] < 2455825])
n_int_nites[igap] = 1
n_int_nites[igap + 1] = 1
n_int_nites[-1] = 1
# Interpolate 9/10 points for kpno data.
n_int_kpno = np.zeros_like(lc_kpno['HJD'])
n_int_kpno[::10] = 1
n_int_kpno[-1] = 1

# Quadratic limb darkening coefficients for primary star from
# Claret & Bloemen, 2011A&A...529A..75C, J/A+A/529/A75/table-af
# Teff = 6000, logg=4.0, [Fe/H] = -0.5, Met=F
# Using Kepler passband for  "white-light" WASP and NITES passbands
ldc_white = [0.3532, 0.2606]
ldc_iband = [0.2552, 0.2598]
# Quadratic limb darkening coefficients for secondary star from
# Claret & Bloemen, 2011A&A...529A..75C, J/A+A/529/A75/table-af
# Teff = 4000, logg=5.0, [Fe/H] = -0.5, Met=F
ldc_jband = [0.1076, 0.2857]

# Initial values
e = 0.3098
om = 278.85*np.pi/180 
f_s = np.sqrt(e)*np.sin(om)
f_c = np.sqrt(e)*np.cos(om)
r_1 = 0.0534
r_2 = 0.0081
incl = 89.084
q = 0.1968 
sbratio_jband = 0.346  # From a previous run
K_1 = 15.84
p_0 = [r_1, r_2, incl, f_s, f_c, sbratio_jband, K_1]
x_e = e + 0.0005*np.random.randn(1000)
x_om = om + 1.29*np.random.randn(1000)*np.pi/180
e_f_s = np.std(np.sqrt(x_e)*np.cos(x_om))
e_f_c = np.std(np.sqrt(x_e)*np.sin(x_om))
# Errors on parameters  - used to set size of initial walkers.
e_p = [0.0021, 0.0004, 0.037, e_f_s, e_f_c, 0.013, 0.01]

# Add estimate of jitter to standard errors for radvel data
jitter = 0.02
radvel['e_RV'] = np.sqrt(radvel['e_RV']**2 + jitter**2)

# Adjust WASP photometry errors
lc_wasp['e_dmag'] = lc_wasp['e_dmag']* 0.84

# Affine-invariant MCMC
ndim, nwalkers, nruns = len(p_0), 50, 1000
pos = [p_0 + e_p*np.random.randn(ndim) for i in range(nwalkers)]
print('Starting MCMC parameter search')
sampler = emcee.EnsembleSampler(nwalkers, ndim, lnlike, 
    args=(lc_wasp,lc_nites,lc_oed,lc_byu,lc_kpno,radvel,
          period, t_zero, q, ldc_white, ldc_iband, ldc_jband, 
          n_int_nites, n_int_kpno),threads=4)
sampler.run_mcmc(pos, nruns)
print('Completed MCMC parameter search')

np.save('chain',sampler.chain)

samples = sampler.chain[:, nruns/2:, :].reshape((-1, ndim))
r_1 = np.median(samples[:,0])
r_2 = np.median(samples[:,1])
incl= np.median(samples[:,2])
f_s = np.median(samples[:,3])
f_c = np.median(samples[:,4])
S_J = np.median(samples[:,5])
K_1 = np.median(samples[:,6])

print('r_1 = ',r_1 ,' +/- ',np.std(samples[:,0]))
print('r_2 = ',r_2 ,' +/- ',np.std(samples[:,1]))
print('incl= ',incl,' +/- ',np.std(samples[:,2]))
print('f_s = ',f_s ,' +/- ',np.std(samples[:,3]))
print('f_c = ',f_c ,' +/- ',np.std(samples[:,4]))
print('S_J = ',S_J ,' +/- ',np.std(samples[:,5]))
print('K_1 = ',K_1 ,' +/- ',np.std(samples[:,6]))
acor = sampler.acor
print('r_1  autocorrelation = ',acor[0])
print('r_2  autocorrelation = ',acor[1])
print('incl autocorrelation = ',acor[2])
print('f_s  autocorrelation = ',acor[3])
print('f_c  autocorrelation = ',acor[4])
print('S_J  autocorrelation = ',acor[5])
print('K_1  autocorrelation = ',acor[6])

af = sampler.acceptance_fraction
print('Median acceptance fraction =',np.median(af))
print('Range of  acceptance fraction =',np.min(af),' to ', np.max(af))
# Plot fit to lightcurves
fig=plt.figure(2,figsize=(5,7))

ph_mod = np.arange(-0.024,0.025,0.001)
offstep = 0.08
offset = 0 

ph_wasp = (1 + (((lc_wasp['HJD']-t_zero)/period) % 1)) % 1
plt.scatter(ph_wasp   ,lc_wasp['dmag'],s=1)
plt.scatter(ph_wasp-1,lc_wasp['dmag'],s=1)
plt.xlim([-0.024,0.024])
plt.ylim([0.35,-0.03])
m = -2.5*np.log10(ellc.lc(ph_mod*period+t_zero, radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_white,
      period=period, t_zero=t_zero, grid_1='sparse',grid_2='sparse' ))
plt.plot(ph_mod,m)
plt.text(-0.023,-0.019,'WASP')

offset += offstep
ph_nites = (1 + (((lc_nites['HJD']-t_zero)/period) % 1)) % 1
plt.scatter(ph_nites   ,lc_nites['dmag']+offset,s=1)
plt.scatter(ph_nites-1,lc_nites['dmag']+offset,s=1)
m = -2.5*np.log10(ellc.lc(ph_mod*period+t_zero, radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_white,
      period=period, t_zero=t_zero , grid_1='sparse',grid_2='sparse'))
plt.plot(ph_mod,m+offset)
plt.text(-0.023,offset-0.019,'NITES')

offset += offstep
ph_oed = (1 + (((lc_oed['HJD']-t_zero)/period) % 1)) % 1
plt.scatter(ph_oed   ,lc_oed['dmag']+offset,s=1)
plt.scatter(ph_oed-1,lc_oed['dmag']+offset,s=1)
m = -2.5*np.log10(ellc.lc(ph_mod*period+t_zero, radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_iband,
      period=period, t_zero=t_zero , grid_1='sparse',grid_2='sparse'))
plt.plot(ph_mod,m+offset)
plt.text(-0.023,offset-0.019,'OED')

offset += offstep
ph_byu = (1 + (((lc_byu['HJD']-t_zero)/period) % 1)) % 1
plt.scatter(ph_byu   ,lc_byu['dmag']+offset,s=1)
plt.scatter(ph_byu-1,lc_byu['dmag']+offset,s=1)
m = -2.5*np.log10(ellc.lc(ph_mod*period+t_zero, radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=0,f_s=f_s, f_c=f_c, ld_1='quad', ldc_1 = ldc_iband,
      period=period, t_zero=t_zero , grid_1='sparse',grid_2='sparse'))
plt.plot(ph_mod,m+offset)
plt.text(-0.023,offset-0.019,'BYU')

offset += offstep
ph_off = 0.533
ph_kpno = (1 + (((lc_kpno['HJD']-t_zero)/period) % 1)) % 1
plt.scatter(ph_kpno-ph_off,lc_kpno['dmag']+offset,s=1)
t = (ph_mod+ph_off)*period+t_zero
m = -2.5*np.log10(ellc.lc(t, radius_1=r_1, radius_2=r_2,
      incl=incl, sbratio=sbratio_jband,f_s=f_s, f_c=f_c, ld_1='quad', 
      ldc_1 = ldc_jband, period=period, t_zero=t_zero ,
      grid_1='sparse',grid_2='sparse'))
plt.plot(ph_mod,m+offset)
plt.text(-0.023,offset-0.019,'KPNO (Phase$-$0.533)')

plt.xlabel("Phase")
plt.ylabel("Magnitude")
plt.locator_params(axis = 'x', nbins = 6)
plt.tight_layout()

if args.eps:
  fig.savefig("J0113+31.eps")
else:
  plt.show()
plt.clf()
plt.close()

# Plot parameter distributions
fig_p = corner.corner(samples, 
  labels=['$r_1$','$r_2$','$i [^\circ]$','$f_c$','$f_s$','$S_J$',
          '$K_1$ [km/s]'],label_kwargs={'fontsize':'18'},
           truths=p_0)
if args.eps:
  fig_p.savefig("corner.eps")
else:
  plt.show()






#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import ellc
import matplotlib.pyplot as plt


lc_dat = np.loadtxt("NightfallCurve.dat")

phase = lc_dat[:,1]
flux_q = 10**(-0.4*lc_dat[:,6])  # 
flux_q = flux_q /np.min(flux_q)

t_zero = 0.5
sbratio = 0.0
r_1 = 0.023765
r_2 = 0.169751
incl = 29.3
q = 0.234
ld_1 = 'quad'
ldc_1 = [0.028,  0.264]
ld_2 = 'quad'
ldc_2 = [0.272, 0.567]
shape_1 = 'roche'
shape_2 = 'roche'
heat_2 = [0.6,3.5,0.0]
lc_1 = ellc.lc(phase,t_zero=t_zero, q=q, heat_2 = heat_2, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, 
    shape_1=shape_1,shape_2=shape_2) 
heat_2 = [1.0 ,1.5, 0.0]
lc_2 = ellc.lc(phase,t_zero=t_zero, q=q, heat_2 = heat_2, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, 
    shape_1=shape_1,shape_2=shape_2) 
heat_2 =  2.1
lc_3 = ellc.lc(phase,t_zero=t_zero, q=q, heat_2 = heat_2, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, 
    shape_1=shape_1,shape_2=shape_2) 

fig=plt.figure(1,figsize=(6,4))
line_q,=plt.plot(phase,flux_q,color='k',label='Nightfall')
line_1,=plt.plot(phase,lc_1,color='g',linestyle='--',label='$0.6,3.5$')
line_2,=plt.plot(phase,lc_2,color='b',linestyle=':',label='$1.0,1.5$')
line_3,=plt.plot(phase,lc_3,color='r',linestyle='-.',label='$\\alpha=2.1$')
plt.xlim([-0.25,0.75])
plt.ylim([0.995,1.065])
plt.xlabel("Phase")
plt.ylabel("Flux")
plt.legend(handles=[line_1,line_2,line_3,line_q],loc='upper right')

plt.tight_layout()

if args.eps:
  fig.savefig("GD448.eps")
else:
  plt.show()



#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import ellc
import matplotlib.pyplot as plt


lc_dat = np.loadtxt("lc.dat")


rho = 0.4
period = 1
a=(3.75226985055)*((period)**(2./3.))*((rho)**(1./3.))
print("a =",a)
r_1 = 1/a
r_2 = 0.1*r_1
ld_1 = 'quad'
ldc_1 = [0.4,0.3]
t_zero=0.25
rotfac_1 = 1/23.9
spots_1 = [[180-360*t_zero*rotfac_1,120-360*t_zero*rotfac_1],
           [0,0],
           [5.739170477266787, 8.11641272572196],
           [0.5,0.5]]
      
flux = ellc.lc(lc_dat[:,0],t_zero=t_zero, period=period, \
    radius_1=r_1, radius_2=r_2,incl=90,sbratio=0, rotfac_1 = rotfac_1, \
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='sphere',\
    grid_1='sparse',grid_2='sparse',spots_1=spots_1)

fontsize=12
fig=plt.figure(1,figsize=(12,8))
plt.subplot(211)
plt.xlim([0,24])
plt.scatter(lc_dat[:,0],lc_dat[:,1],color='darkgreen',marker='x',s=2)
plt.plot(lc_dat[:,0],flux,linewidth=1,color='darkblue')
plt.xlabel("Time [d]",fontsize=fontsize)
plt.ylabel("Normalized flux",fontsize=fontsize)
plt.tick_params(axis='both', labelsize=fontsize)
plt.subplot(234)
plt.plot(lc_dat[:,0]-6.25,flux,linewidth=1,color='darkblue')
plt.scatter(lc_dat[:,0]-6.25,lc_dat[:,1],color='darkgreen',marker='x',s=2)
plt.locator_params(axis = 'x', nbins = 4)
plt.xlabel("Time-6.25 [d]",fontsize=fontsize)
plt.ylabel("Normalized flux",fontsize=fontsize)
plt.xlim([-0.08,0.08])
plt.ylim([0.975,0.995])
plt.tick_params(axis='both', labelsize=fontsize)
plt.subplot(235)
plt.scatter(lc_dat[:,0]-9.25,lc_dat[:,1],color='darkgreen',marker='x',s=2)
plt.plot(lc_dat[:,0]-9.25,flux,linewidth=1,color='darkblue')
plt.locator_params(axis = 'x', nbins = 4)
plt.xlabel("Time-9.25 [d]",fontsize=fontsize)
plt.xlim([-0.08,0.08])
plt.ylim([0.97,0.99])
plt.tick_params(axis='both', labelsize=fontsize)
plt.subplot(236)
plt.scatter(lc_dat[:,0]-14.25,lc_dat[:,1],color='darkgreen',marker='x',s=2)
plt.plot(lc_dat[:,0]-14.25,flux,linewidth=1,color='darkblue')
plt.locator_params(axis = 'x', nbins = 4)
plt.xlabel("Time-14.25 [d]",fontsize=fontsize)
plt.xlim([-0.08,0.08])
plt.ylim([0.98, 1.0])
plt.tick_params(axis='both', labelsize=fontsize)
plt.tight_layout()

if args.eps:
  fig.savefig("KSint.eps")
else:
  plt.show()






# In this version ...
#  - checking for valid spot parameters
#  - Up to 3 spots on star 2
#  - init-only option
#  - Added sloan filters to limb darkening
#  - Fixed python3 prior checking bug
#
from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import numpy as np
from ellc import ldy,lc
import os
import emcee
from six.moves import input
import matplotlib.pyplot as plt
from datetime import datetime
from astropy.table import Table, Column
from time import time as time_
import sys

def get_varpar_value(var_name, varpar_v, varpar_l):
  for i,s in enumerate(varpar_l):
    if s == var_name:
      return varpar_v[i]
  raise IndexError

def _detrending_coeffs(basis_funcs, residuals, weights, return_errors=False):
  # Calculate detrening coefficients for a given set of residuals
  A = basis_funcs*np.transpose([weights])
  b = residuals*weights
  scale = np.sqrt(np.square(A).sum(0))
  scale[scale == 0] = 1
  A_s = A/scale
  c_s,resid,rank,sval = np.linalg.lstsq(A_s,b)
  c = c_s/scale
  if return_errors :
    V = np.linalg.inv(np.dot(A_s.T,A_s))
    V /= np.core.numeric.outer(scale,scale)
    fac = resid / (len(residuals) - len(c) - 2.0)
    e_c = np.sqrt(np.diagonal(V*fac))
    return c,e_c
  else:
    return c


def _lnlike(varpar_v, lcdata, basis_funcs, varpar_l, fixpar_d, priors, ldy_, 
    n_spot_1, n_spot_2, grid_size, return_fit=False):

  try:
    r_1 = get_varpar_value('r_1',varpar_v, varpar_l)
  except:
    r_1 = fixpar_d['r_1']
  if (r_1 <= 0) or (r_1 > 1) : return -np.inf

  try:
    r_2 = get_varpar_value('r_2',varpar_v, varpar_l)
  except:
    r_2 = fixpar_d['r_2']
  if (r_2 <= 0) or (r_2 > 1) : return -np.inf

  try:
    sb2 = get_varpar_value('sb2',varpar_v, varpar_l)
  except:
    sb2 = fixpar_d['sb2']
  if sb2 < 0 : return -np.inf

  try:
    incl = get_varpar_value('incl',varpar_v, varpar_l)
  except:
    incl = fixpar_d['incl']
  if (incl <= 0) or (incl > 90) : return -np.inf

  try:
    l_3 = get_varpar_value('l_3',varpar_v, varpar_l)
  except:
    l_3 = fixpar_d['l_3']

  try:
    t0 = get_varpar_value('T_0',varpar_v, varpar_l)
  except:
    t0 = fixpar_d['T_0']

  try:
    period = get_varpar_value('P',varpar_v, varpar_l)
  except:
    period = fixpar_d['P']

  try:
    f_c = get_varpar_value('f_c',varpar_v, varpar_l)
  except:
    f_c = fixpar_d['f_c']
  if (f_c <= -1) or (f_c >= 1): return -np.inf

  try:
    f_s = get_varpar_value('f_s',varpar_v, varpar_l)
  except:
    f_s = fixpar_d['f_s']
  if (f_s <= -1) or (f_s >= 1): return -np.inf

  try:
    domdt = get_varpar_value('domdt',varpar_v, varpar_l)
  except:
    domdt = fixpar_d['domdt']
  if (domdt <= -1) or (domdt >= 1): return -np.inf

  try:
    frot1 = get_varpar_value('F_1',varpar_v, varpar_l)
  except:
    frot1 = fixpar_d['F_1']

  try:
    frot2 = get_varpar_value('F_2',varpar_v, varpar_l)
  except:
    frot2 = fixpar_d['F_2']

  try:
    t1 = get_varpar_value('Teff1',varpar_v, varpar_l)
  except:
    t1 = fixpar_d['Teff1']

  try:
    g1 = get_varpar_value('logg1',varpar_v, varpar_l)
  except:
    g1 = fixpar_d['logg1']

  try:
    t2 = get_varpar_value('Teff2',varpar_v, varpar_l)
  except:
    t2 = fixpar_d['Teff2']

  try:
    g2 = get_varpar_value('logg2',varpar_v, varpar_l)
  except:
    g2 = fixpar_d['logg2']

  try:
    m = get_varpar_value('[M/H]',varpar_v, varpar_l)
  except:
    m = fixpar_d['[M/H]']

  try:
    A_1 = get_varpar_value('A_1',varpar_v, varpar_l)
  except:
    A_1 = fixpar_d['A_1']
  if A_1 < 0 : return -np.inf

  try:
    A_2 = get_varpar_value('A_2',varpar_v, varpar_l)
  except:
    A_2 = fixpar_d['A_2']
  if A_2 < 0 : return -np.inf

  try:
    sig_ext=get_varpar_value('sig_ext',varpar_v, varpar_l)
  except:
    sig_ext=fixpar_d['sig_ext']
  if sig_ext < 0 : return -np.inf


  spots_1 = None
  if n_spot_1 > 0:
    try:
      s_1_1 = get_varpar_value('s_1_1',varpar_v, varpar_l)
    except:
      s_1_1 = fixpar_d['s_1_1']
    if (s_1_1 < 0) or (s_1_1 >= 90) : return -np.inf
    try:
      l_1_1 = get_varpar_value('l_1_1',varpar_v, varpar_l)
    except:
      l_1_1 = fixpar_d['l_1_1']
    try:
      b_1_1 = get_varpar_value('b_1_1',varpar_v, varpar_l)
    except:
      b_1_1 = fixpar_d['b_1_1']
    if (b_1_1 < -90) or (b_1_1 > 90) : return -np.inf
    try:
      f_1_1 = get_varpar_value('f_1_1',varpar_v, varpar_l)
    except:
      f_1_1 = fixpar_d['f_1_1']
    if f_1_1 <0: return -np.inf
    spots_1 = [[l_1_1],[b_1_1],[s_1_1],[f_1_1]]

  if n_spot_1 > 1:
    try:
      s_2_1 = get_varpar_value('s_2_1',varpar_v, varpar_l)
    except:
      s_2_1 = fixpar_d['s_2_1']
    if (s_2_1 < 0) or (s_2_1 >= 90) : return -np.inf
    try:
      l_2_1 = get_varpar_value('l_2_1',varpar_v, varpar_l)
    except:
      l_2_1 = fixpar_d['l_2_1']
    try:
      b_2_1 = get_varpar_value('b_2_1',varpar_v, varpar_l)
    except:
      b_2_1 = fixpar_d['b_2_1']
    if (b_2_1 < -90) or (b_2_1 > 90) : return -np.inf
    try:
      f_2_1 = get_varpar_value('f_2_1',varpar_v, varpar_l)
    except:
      f_2_1 = fixpar_d['f_2_1']
    if f_2_1 <0: return -np.inf
    spots_1 = [[l_1_1,l_2_1],[b_1_1,b_2_1],[s_1_1,s_2_1],[f_1_1,f_2_1]]

  spots_2 = None
  if n_spot_2 > 0:
    try:
      s_1_2 = get_varpar_value('s_1_2',varpar_v, varpar_l)
    except:
      s_1_2 = fixpar_d['s_1_2']
    if (s_1_2 < 0) or (s_1_2 >= 90) : return -np.inf
    try:
      l_1_2 = get_varpar_value('l_1_2',varpar_v, varpar_l)
    except:
      l_1_2 = fixpar_d['l_1_2']
    try:
      b_1_2 = get_varpar_value('b_1_2',varpar_v, varpar_l)
    except:
      b_1_2 = fixpar_d['b_1_2']
    if (b_1_2 < -90) or (b_1_2 > 90) : return -np.inf
    try:
      f_1_2 = get_varpar_value('f_1_2',varpar_v, varpar_l)
    except:
      f_1_2 = fixpar_d['f_1_2']
    if f_1_2 <0: return -np.inf
    spots_2 = [[l_1_2],[b_1_2],[s_1_2],[f_1_2]]

  if n_spot_2 > 1:
    try:
      s_2_2 = get_varpar_value('s_2_2',varpar_v, varpar_l)
    except:
      s_2_2 = fixpar_d['s_2_2']
    if (s_2_2 < 0) or (s_2_2 >= 90) : return -np.inf
    try:
      l_2_2 = get_varpar_value('l_2_2',varpar_v, varpar_l)
    except:
      l_2_2 = fixpar_d['l_2_2']
    try:
      b_2_2 = get_varpar_value('b_2_2',varpar_v, varpar_l)
    except:
      b_2_2 = fixpar_d['b_2_2']
    if (b_2_2 < -90) or (b_2_2 > 90) : return -np.inf
    try:
      f_2_2 = get_varpar_value('f_2_2',varpar_v, varpar_l)
    except:
      f_2_2 = fixpar_d['f_2_2']
    if f_2_2 <0: return -np.inf
    spots_2 = [[l_1_2,l_2_2],[b_1_2,b_2_2],[s_1_2,s_2_2],[f_1_2,f_2_2]]

  if n_spot_2 > 2:
    try:
      s_3_2 = get_varpar_value('s_3_2',varpar_v, varpar_l)
    except:
      s_3_2 = fixpar_d['s_3_2']
    if (s_3_2 < 0) or (s_3_2 >= 90) : return -np.inf
    try:
      l_3_2 = get_varpar_value('l_3_2',varpar_v, varpar_l)
    except:
      l_3_2 = fixpar_d['l_3_2']
    try:
      b_3_2 = get_varpar_value('b_3_2',varpar_v, varpar_l)
    except:
      b_3_2 = fixpar_d['b_3_2']
    if (b_3_2 < -90) or (b_3_2 > 90) : return -np.inf
    try:
      f_3_2 = get_varpar_value('f_3_2',varpar_v, varpar_l)
    except:
      f_3_2 = fixpar_d['f_3_2']
    if f_3_2 <0: return -np.inf
    spots_2 = [[l_1_2,l_2_2,l_3_2],
               [b_1_2,b_2_2,b_3_2],
               [s_1_2,s_2_2,s_3_2],
               [f_1_2,f_2_2,f_3_2]]



  a1,a2,a3,a4,y1 = ldy_(t1,g1,m)
  if np.isnan(y1):
    return -np.inf
  ldc_1 = [a1,a2,a3,a4]
  a1,a2,a3,a4,y2 = ldy_(t2,g2,m)
  if np.isnan(y2):
    return -np.inf
  ldc_2 = [a1,a2,a3,a4]

  heat_1 = A_1
  heat_2 = A_2

  q = fixpar_d['q']
  t_exp = fixpar_d['t_exp']
  if (t_exp <= 0) or (np.max(lcdata['flag']) < 2):
    t_exp = None

  # Convenience labels for data to be fit
  mask_fit = lcdata['flag'] >= 0
  mag_obs_fit = (lcdata[mask_fit])['mag']
  err_obs_fit = (lcdata[mask_fit])['e_mag']
  set_obs_fit = (lcdata[mask_fit])['subset']
  wt_obs_fit = 1./(err_obs_fit**2 + sig_ext**2)

  # Return model normalized to subset 0 only
  if return_fit:
    n_int = np.array(lcdata['flag'])
    n_int[n_int<0] = 1 
    t_obs = lcdata['time']
    flux_mod = lc(t_obs, radius_1=r_1, radius_2=r_2, sbratio=sb2, incl=incl,
        light_3=l_3, t_zero=t0, period=period, q=q, f_c=f_c, f_s=f_s,
        domdt=domdt, rotfac_1=frot1, rotfac_2=frot2,
        heat_1=heat_1, heat_2=heat_2,
        n_int=n_int, ld_1='claret', ldc_1=ldc_1, ld_2='claret', ldc_2=ldc_2,
        gdc_1=y1, gdc_2=y2, t_exp=t_exp, grid_1=grid_size, grid_2=grid_size,
        shape_1='roche', shape_2='roche',spots_1=spots_1,spots_2=spots_2)
    mag_mod = -2.5*np.log10(flux_mod)
    mag_mod_fit = mag_mod[mask_fit]
    mask_fit_0 = np.logical_and(set_obs_fit == 0, np.isnan(mag_mod_fit) == False) 
    mag_obs_fit_0 = mag_obs_fit[mask_fit_0]
    mag_mod_fit_0 = mag_mod_fit[mask_fit_0]
    wt_obs_fit_0 = wt_obs_fit[mask_fit_0]
    zp_0 = np.average(mag_obs_fit_0-mag_mod_fit_0,weights=wt_obs_fit_0)
    mag_mod_0 = mag_mod + zp_0
    return mag_mod_0

  # Check for input value outside range set by priors
  for p in priors:
    p_type = p['p_type'].decode()
    if p_type == 'U':
      p_var = p['p_var'].decode()
      if p_var in varpar_l:
        v = get_varpar_value(p_var, varpar_v, varpar_l)
      elif p_var == 'e':
        v = f_c**2 + f_s**2
      elif p_var == 'om':
        v = np.arctan2(f_s,f_c)
      elif p_var == 'rsum':
        v = r_1+r_2
      elif p_var == 'lrat':
        v = sb2*(r_2/r_1)**2
      elif p_var == 'k':
        v = r_2/r_1
      else:
        raise Exception('No such prior variable',p['p_var'])

      if (v < p['p_par1']) or (v > p['p_par2']):
        return -np.inf

  t_fit = (lcdata[mask_fit])['time']
  n_int_fit = (lcdata[mask_fit])['flag']
  flux_mod_fit = lc(t_fit, radius_1=r_1, radius_2=r_2, sbratio=sb2, incl=incl, 
      light_3=l_3, t_zero=t0, period=period, q=q, f_c=f_c, f_s=f_s, domdt=domdt,
      rotfac_1=frot1, rotfac_2=frot2,heat_1=heat_1, heat_2=heat_2,
      n_int=n_int_fit,ld_1='claret', ldc_1=ldc_1, ld_2='claret', ldc_2=ldc_2,
      gdc_1=y1, gdc_2=y2, t_exp=t_exp, grid_1=grid_size, grid_2=grid_size,
      shape_1='roche', shape_2='roche',spots_1=spots_1,spots_2=spots_2)

  if (True in np.isnan(flux_mod_fit)) or np.min(flux_mod_fit) <= 0 :
    return -np.inf

  mag_mod_fit = -2.5*np.log10(flux_mod_fit)
  mask_fit_0 = (set_obs_fit == 0)  
  mag_obs_fit_0 = mag_obs_fit[mask_fit_0]
  mag_mod_fit_0 = mag_mod_fit[mask_fit_0]
  wt_obs_fit_0 = wt_obs_fit[mask_fit_0]
  zp_0 = np.average(mag_obs_fit_0-mag_mod_fit_0,weights=wt_obs_fit_0)
  mag_mod_fit = mag_mod_fit + zp_0
  res_obs_fit = mag_obs_fit - mag_mod_fit
  if basis_funcs is None:
    mag_fit_fit = mag_mod_fit 
  else:
    basis_funcs_fit = basis_funcs[mask_fit]
    c = _detrending_coeffs(basis_funcs_fit,res_obs_fit,wt_obs_fit)
    mag_fit_fit = mag_mod_fit + np.dot(basis_funcs_fit,c)

  # The term 0.5*np.sum(np.log(wt_obs_fit)) accounts for the variation in
  # the standard errors # sigma = sqrt(sig_int^2+sig_ext^2).
  # See Ford, E.B., 2006ApJ...642..505F, equations (12) and (15)
  lnlike = 0.5*(np.sum(np.log(wt_obs_fit)) -
      np.sum(wt_obs_fit*(mag_fit_fit-mag_obs_fit)**2)) 
      

  for p in priors:
    p_type = p['p_type'].decode()
    if p_type  == 'G':
      p_var = p['p_var'].decode() 
      if p_var in varpar_l:
        v = get_varpar_value(p_var, varpar_v, varpar_l)
      elif p_var == 'e':
        v = f_c**2 + f_s**2
      elif p_var == 'om':
        v = np.arctan2(f_s,f_c)
      elif p_var == 'rsum':
        v = r_1+r_2
      elif p_var == 'lrat':
        v = sb2*(r_2/r_1)**2
      elif p_var == 'k':
        v = r_2/r_1
      else:
        raise Exception('No such prior variable')

      lnlike = lnlike -0.5*((v-p['p_par1'])/(p['p_par2']))**2

  return lnlike
 
"""
 Light curve analysis using ellc binary star model and emcee 

 
 The output file is a numpy array containing the parameters at each step of
 the chain with the lnprobability value at 
"""

import argparse
from  astropy.io import fits
import numpy as np
from numpy.lib.recfunctions import merge_arrays

def main():

  datetime_start = datetime.today()
  print("\n Start ellc_emcee.py at {:%c}\n".format(datetime_start))

  print(' Use ./ellc_emcee.py -h to see command line options.')
  print('\n Output file data format set by extension name, e.g. "chain.fits"')
  print(' See astropy.table docmentation for further details.')

  # Set up command line switched
  parser = argparse.ArgumentParser(
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    description='Light curve analysis using ellc binary star model and emcee'
  )
  parser.add_argument("--grid_size", "-g", 
    default='sparse',
    choices=['very_sparse', 'sparse', 'default', 'fine','very_fine'],
    help='Grid size'
  )
  parser.add_argument("--tolerant", "-t", 
    action="store_const",
    dest='tolerant',
    const=True,
    default=False,
    help='Tolerate invalid input'
  )
  parser.add_argument("--overwrite", "-f", 
    action="store_const",
    dest='overwrite',
    const=True,
    default=False,
    help='Force overwrite of existing output files.'
  )
  parser.add_argument("--log_file", "-l", 
    default='ellc_emcee.log',
    help='Log file name'
  )

  parser.add_argument("--init-only", "-i", 
    action="store_const",
    dest='init_only',
    const=True,
    default=False,
    help='Write only the initial model'
  )

  parser.add_argument("--subsets", "-s", 
    action="store_const",
    dest='subsets',
    const=True,
    default=False,
    help='Normalize individual subsets identified in input file'
  )

  parser.add_argument("--no-progress-meter", "-n", 
    action="store_const",
    dest='nometer',
    const=True,
    default=False,
    help='Do not print progress meter to screen'
  )

  parser.add_argument("--chain_file", "-c", 
    default='chain.csv',
    help='Output file for chain data'
  )

  parser.add_argument("--model_file", "-m", 
    default='model.csv',
    help='Output file for the best fit model light curve.'
  )

  # Get command line options
  args = parser.parse_args()

  print(" Opening log file",args.log_file)
  if (args.overwrite):
    flags = os.O_CREAT | os.O_WRONLY
  else:
    flags = os.O_CREAT | os.O_EXCL | os.O_WRONLY

  try:
    file_handle = os.open(args.log_file,flags)
  except:
    raise

  f_log = os.fdopen(file_handle, 'w')

  print("\n Start ellc_emcee.py at {:%c}".format(datetime_start),file=f_log)

  print("\n Using ellc grid size",args.grid_size)
  print("\n Using ellc grid size",args.grid_size,file=f_log)

  print('\n Entering user options and parameter values.')
  print(' Blank lines and comments following "#" on input are ignored.')

  if (args.subsets):
    print("\n Light curve data file must be an ascii file with five columns of")
    print(" time, magnitude, error, flag and subset in that order.\n")
    print(" Time in days")
    print(" Offset of each subset (identified by an integer) relative to")
    print(" subset 0 (which must be present) is calculated at each iteration ")
    print(" and is applied to output lightcurve.") 
  else:
    print("\n Light curve data file must be an ascii file with four columns of")
  print(' Comment lines starting with "#" and additional columns are ignored.')
  print(' Flag is an integer used to set the following options.')
  print(' >= 0 to include observation in fit.')
  print(' > 1 to set no. of integration points to account for exposure time')
  print(' = 0 to interpolate the point from neighbouring values.')
  print(' < 0 to exclude point from fit, e.g., a point for plotting only.\n')

  prompt = "Enter light curve file name : "
  while True:
    line = input(prompt)
    line = line.lstrip()
    if (len(line) == 0) and args.tolerant: continue   # Skip blank lines
    if line[0] == '#': continue   # Skip comment lines starting #
    line = (line.split('#'))[0]   # and ignore text after # on data lines

    lcfile = (line.split())[0]
    print("\n Attempting to read light curve file ",lcfile)
    print("\n Attempting to read light curve file ",lcfile,file=f_log)
    if (args.subsets):
      names = (str('time'),str('mag'),str('e_mag'),str('flag'),str('subset'))
      formats = ('f8','f4','f4','i4','i4')
    else:
      names = (str('time'),str('mag'),str('e_mag'),str('flag'))
      formats = ('f8','f4','f4','i4')
    try:
      lcdata = np.loadtxt(lcfile,dtype={'names': (names), 'formats': (formats)})
    except:
        if args.tolerant: 
          print ('Failed to read light curve data from file',lcfile)
          continue
        else:
          raise
    break
      
  print(' Succesfully read light curve file',lcfile)
  print(' Succesfully read light curve file',lcfile,file=f_log)
  n_data = len(lcdata)
  print(' Line  1:',lcdata[0])
  print(' Line {}:'.format(n_data),lcdata[n_data-1])
  print(' Line  1:',lcdata[0],file=f_log)
  print(' Line {}:'.format(n_data),lcdata[n_data-1],file=f_log)

  if (args.subsets):
    n_subsets = len(np.unique(lcdata['subset']))
    print(' Number of subsets is ',n_subsets)
    print(' Number of subsets is ',n_subsets,file=f_log)
  else:
    n_subsets = 1
    z = np.array(np.zeros(len(lcdata)),dtype=[(str('subset'),str('i4'))])
    lcdata = merge_arrays([lcdata,z],flatten='True')

  if len(lcdata[lcdata['subset'] == 0]) == 0:
      raise Exception ('No data in subset 0 in light curve data file',lcfile)

  print('\n Bands: u,v,b,y (Stromgren)')
  print('        U, B, V, R, I, J, H, K (Johnson)')
  print('        S1, S2, S3, S4 (Spitzer IRAC)')
  print('        Kp (Kepler)')
  print('        C (CoRoT)')

  allowed_bands = ldy.list_bands()

  prompt = 'Enter photometric band code for light curve :'
  while True:
    line = input(prompt)
    line = line.lstrip()
    if (len(line) == 0) and args.tolerant: continue   # Skip blank lines
    if line[0] == '#': continue   # Skip comment lines starting #
    line = (line.split('#'))[0]   # and ignore text after # on data lines

    band = (line.split())[0]
    if band not in allowed_bands:
      if args.tolerant: 
        print ('Photometric band not available ',band)
        continue
      raise Exception ('Photometric band not available ',band)
    break

  print('\n Selected photometric band is',band)
  print('\n#  -- Start of user input block -- ',file=f_log)
  print(lcfile,'   #+ Light curve file ',file=f_log)
  print(band,'   #+ Photometric band ',file=f_log)

  def _get_param(prompt,default,lo_lim,hi_lim,integer=False):
    while True:
      if default is None:
        full_prompt = "{:s} : ".format(prompt)
      else:
        if integer:
          full_prompt = "{:s} [{:d}] : ".format(prompt,default)
        else:
          full_prompt = "{:s} [{:f}] : ".format(prompt,default)
      line = input(full_prompt)
      if len(line) == 0 : 
        if default is None:
          continue
        else:
          print (default,'     #+ ',prompt,' (fixed, default)',file=f_log)
          return default,None
      if line[0] == '#': continue   # Skip comment lines starting #
      line = (line.split('#'))[0]   # and ignore text after # on data lines
      linedata = line.split()
      if len(linedata) == 0 : 
        if default is None:
          continue
        else:
          print (default,'     #+ ',prompt,' (fixed, default)',file=f_log)
          return default,None
      if integer:
        value = np.int(linedata[0])
      else:
        value = np.float(linedata[0])
      if (lo_lim is not None) and (value < lo_lim):
        if args.tolerant:
          print ('Input value exceeds lower limit',lo_lim)
          continue
        raise Exception('Input value exceeds lower limit',lo_lim)
      if (hi_lim is not None) and (value > hi_lim):
        if args.tolerant:
          print ('Input value exceeds upper limit',hi_lim)
          continue
        raise Exception('Input value exceeds upper limit',hi_lim)
      if len(linedata) == 1:
        print (value,'     #+ ',prompt,' (fixed)',file=f_log)
        return value,None
      else:
        error = np.float(linedata[1])
        print (value,error,'     #+ ',prompt,file=f_log)
        return value,error

  print('\n Now enter model parameters with their standard error estimate')
  print(' for free parameters, or value only for fixed parameters.')
  print(' T_eff, logg and [M/H] are only used to find limb darkening values.')
  print(' Hit enter to accept default (fixed) value where shown, e.g., [0].\n')

 # Could probably tidy this up by iterating over list of varaibles - oh well..

  varpar_v = []   # Values of free parameters
  varpar_e = []   # Errors on free parameters
  varpar_l = []   # List of free parameters
  fixpar_d = {}   # Dictionary of fixed parameters and values

  r_1,e_r1 = _get_param('r_1 = R_1/a',None,0.0001,1.0 )
  if e_r1 is None:
    fixpar_d['r_1'] = r_1
  else:
    varpar_v.append(r_1)
    varpar_e.append(e_r1)
    varpar_l.append('r_1')

  r_2,e_r2 = _get_param('r_2 = R_2/a',None,0.0001,1.0)
  if e_r2 is None:
    fixpar_d['r_2'] = r_2
  else:
    varpar_v.append(r_2)
    varpar_e.append(e_r2)
    varpar_l.append('r_2')

  sbrat,e_sbrat = _get_param('s = S_2/S_1, surface brightness ratio',
      None,0.0,None)
  if e_sbrat is None:
    fixpar_d['sb2'] = sbrat
  else:
    varpar_v.append(sbrat)
    varpar_e.append(e_sbrat)
    varpar_l.append('sb2')

  incl,e_incl = _get_param('i = inclination (degrees)',None,0,90)
  if e_incl is None:
    fixpar_d['incl'] = incl
  else:
    varpar_v.append(incl)
    varpar_e.append(e_incl)
    varpar_l.append('incl')

  l_3,e_l_3 = _get_param('l_3 = third light',0,0,None)
  if e_l_3 is None:
    fixpar_d['l_3'] = l_3
  else:
    varpar_v.append(l_3)
    varpar_e.append(e_l_3)
    varpar_l.append('l_3')

  tzero,e_tzero = _get_param('T_0, time of mid-eclipse',None,None,None)
  if e_tzero is None:
    fixpar_d['T_0'] = tzero
  else:
    varpar_v.append(tzero)
    varpar_e.append(e_tzero)
    varpar_l.append('T_0')

  per,e_per = _get_param('P = orbital period (days)',None,0,None)
  if e_per is None:
    fixpar_d['P'] = per
  else:
    varpar_v.append(per)
    varpar_e.append(e_per)
    varpar_l.append('P')

  f_c,e_f_c = _get_param('f_c = sqrt(e).cos(omega)',0,-0.999,0.999)
  if e_f_c is None:
    fixpar_d['f_c'] = f_c
  else:
    varpar_v.append(f_c)
    varpar_e.append(e_f_c)
    varpar_l.append('f_c')

  f_s,e_f_s = _get_param('f_s = sqrt(e).sin(omega)',0,-0.999,0.999)
  if e_f_s is None:
    fixpar_d['f_s'] = f_s
  else:
    varpar_v.append(f_s)
    varpar_e.append(e_f_s)
    varpar_l.append('f_s')

  domdt,e_domdt = _get_param('domdt = apsidal motion rate [deg/cycle])',0,-1,1) 
  if e_domdt is None:
    fixpar_d['domdt'] = domdt
  else:
    varpar_v.append(domdt)
    varpar_e.append(e_domdt)
    varpar_l.append('domdt')

  frot1,e_frot1 = _get_param('F_1 = asynchronous rotation factor, star 1',
      1,None,None)
  if e_frot1 is None:
    fixpar_d['F_1'] = frot1
  else:
    varpar_v.append(frot1)
    varpar_e.append(e_frot1)
    varpar_l.append('F_1')

  frot2,e_frot2 = _get_param('F_2 = asynchronous rotation factor, star 2',
      1,None,None)
  if e_frot2 is None:
    fixpar_d['F_2'] = frot2
  else:
    varpar_v.append(frot2)
    varpar_e.append(e_frot2)
    varpar_l.append('F_2')

  teff1,e_teff1 = _get_param('Teff1 = T_eff,1',None,3500,50000)
  if e_teff1 is None:
    fixpar_d['Teff1'] = teff1
  else:
    varpar_v.append(teff1)
    varpar_e.append(e_teff1)
    varpar_l.append('Teff1')

  logg1,e_logg1 = _get_param('g1 = log(g_1)',4.0,0.0,5.0)
  if e_logg1 is None:
    fixpar_d['logg1'] = logg1
  else:
    varpar_v.append(logg1)
    varpar_e.append(e_logg1)
    varpar_l.append('logg1')

  teff2,e_teff2 = _get_param('Teff2 = T_eff,2',None,3500,50000)
  if e_teff2 is None:
    fixpar_d['Teff2'] = teff2
  else:
    varpar_v.append(teff2)
    varpar_e.append(e_teff2)
    varpar_l.append('Teff2')

  logg2,e_logg2 = _get_param('g2 = log(g_1)',4.0,0.0,5.0)
  if e_logg2 is None:
    fixpar_d['logg2'] = logg2
  else:
    varpar_v.append(logg2)
    varpar_e.append(e_logg2)
    varpar_l.append('logg2')

  M_H,e_M_H = _get_param('[M/H]',0.0,-5.0,1.0)
  if e_M_H is None:
    fixpar_d['[M/H]'] = M_H
  else:
    varpar_v.append(M_H)
    varpar_e.append(e_M_H)
    varpar_l.append('[M/H]')

  A_1,e_A_1 = _get_param('A_1 = reflection coefficient, star 1',0,0,None)
  if e_A_1 is None:
    fixpar_d['A_1'] = A_1
  else:
    varpar_v.append(A_1)
    varpar_e.append(e_A_1)
    varpar_l.append('A_1')

  A_2,e_A_2 = _get_param('A_2 = reflection coefficient, star 2',0,0,None)
  if e_A_2 is None:
    fixpar_d['A_2'] = A_2
  else:
    varpar_v.append(A_2)
    varpar_e.append(e_A_2)
    varpar_l.append('A_2')

  sig_ext,e_sig_ext = _get_param('sig_ext = external noise',0,0,None)
  if e_sig_ext is None:
    fixpar_d['sig_ext'] = sig_ext
  else:
    varpar_v.append(sig_ext)
    varpar_e.append(e_sig_ext)
    varpar_l.append('sig_ext')

  print('\n Now enter fix parameters for the model. ')

  n_poly,dummy = _get_param('Polynomial degree for normalization',1,0,8,
      integer=True)  

  q,dummy = _get_param('Mass ratio q=m_2/m_1',1,0,None)  
  fixpar_d['q'] = q


  t_exp,dummy = _get_param('T_exp (seconds)',0,0,None)  
  fixpar_d['t_exp'] = t_exp/86400.

  print('\n Number of spots per star is limited to 2. ')

  allowed_prior_vars = ['e','om','r_1','r_2','rsum','k','sb2','incl','l_3',
      'lrat','T_0','P', 'f_c', 'f_s', 'domdt', 'F_1', 'F_2', 'A_1', 'A_2', 
      'sig_ext', 'Teff1', 'logg1', 'Teff2', 'logg2', '[M/H]'] 

  n_spot_1,dummy = _get_param('Number of spots on star 1',0,0,2,integer=True)  
  if n_spot_1 > 0:
    s_1_1,e_s_1_1 = _get_param('s_1_1 = size (degrees) of spot 1, star 1',
        None,0.001,60)  
    if e_s_1_1 is None:
      fixpar_d['s_1_1'] = s_1_1
    else:
      varpar_v.append(s_1_1)
      varpar_e.append(e_s_1_1)
      varpar_l.append('s_1_1')
      allowed_prior_vars.append('s_1_1')
    l_1_1,e_l_1_1 = _get_param('l_1_1 = longitude (degrees) of spot 1, star 1',
        None,-360,360)  
    if e_l_1_1 is None:
      fixpar_d['l_1_1'] = l_1_1
    else:
      varpar_v.append(l_1_1)
      varpar_e.append(e_l_1_1)
      varpar_l.append('l_1_1')
      allowed_prior_vars.append('l_1_1')
    b_1_1,e_b_1_1 = _get_param('b_1_1 = latitude (degrees) of spot 1, star 1',
        None,-90,90)
    if e_b_1_1 is None:
      fixpar_d['b_1_1'] = b_1_1
    else:
      varpar_v.append(b_1_1)
      varpar_e.append(e_b_1_1)
      varpar_l.append('b_1_1')
      allowed_prior_vars.append('b_1_1')
    f_1_1,e_f_1_1 = _get_param('f_1_1 = brightness factor of spot 1, star 1',
        None,0.0,None)  
    if e_f_1_1 is None:
      fixpar_d['f_1_1'] = f_1_1
    else:
      varpar_v.append(f_1_1)
      varpar_e.append(e_f_1_1)
      varpar_l.append('f_1_1')
      allowed_prior_vars.append('f_1_1')

  if n_spot_1 > 1:
    s_2_1,e_s_2_1 = _get_param('s_2_1 = size (degrees) of spot 1, star 1',
        None,0.001,60)  
    if e_s_2_1 is None:
      fixpar_d['s_2_1'] = s_2_1
    else:
      varpar_v.append(s_2_1)
      varpar_e.append(e_s_2_1)
      varpar_l.append('s_2_1')
      allowed_prior_vars.append('s_2_1')
    l_2_1,e_l_2_1 = _get_param('l_2_1 = longitude (degrees) of spot 1, star 1',
        None,-360,360)  
    if e_l_2_1 is None:
      fixpar_d['l_2_1'] = l_2_1
    else:
      varpar_v.append(l_2_1)
      varpar_e.append(e_l_2_1)
      varpar_l.append('l_2_1')
      allowed_prior_vars.append('l_2_1')
    b_2_1,e_b_2_1 = _get_param('b_2_1 = latitude (degrees) of spot 1, star 1',
        None,-90,90)
    if e_b_2_1 is None:
      fixpar_d['b_2_1'] = b_2_1
    else:
      varpar_v.append(b_2_1)
      varpar_e.append(e_b_2_1)
      varpar_l.append('b_2_1')
      allowed_prior_vars.append('b_2_1')
    f_2_1,e_f_2_1 = _get_param('f_2_1 = brightness factor of spot 1, star 1',
        None,0.0,None)  
    if e_f_2_1 is None:
      fixpar_d['f_2_1'] = f_2_1
    else:
      varpar_v.append(f_2_1)
      varpar_e.append(e_f_2_1)
      varpar_l.append('f_2_1')
      allowed_prior_vars.append('f_2_1')
   

  n_spot_2,dummy = _get_param('Number of spots on star 2',0,0,3,integer=True)  
  if n_spot_2 > 0:
    s_1_2,e_s_1_2 = _get_param('s_1_2 = size (degrees) of spot 1, star 2',
        None,0.001,60)  
    if e_s_1_2 is None:
      fixpar_d['s_1_2'] = s_1_2
    else:
      varpar_v.append(s_1_2)
      varpar_e.append(e_s_1_2)
      varpar_l.append('s_1_2')
      allowed_prior_vars.append('s_1_2')
    l_1_2,e_l_1_2 = _get_param('l_1_2 = longitude (degrees) of spot 1, star 2',
        None,-360,360)  
    if e_l_1_2 is None:
      fixpar_d['l_1_2'] = l_1_2
    else:
      varpar_v.append(l_1_2)
      varpar_e.append(e_l_1_2)
      varpar_l.append('l_1_2')
      allowed_prior_vars.append('l_1_2')
    b_1_2,e_b_1_2 = _get_param('b_1_2 = latitude (degrees) of spot 1, star 2',
        None,-90,90)
    if e_b_1_2 is None:
      fixpar_d['b_1_2'] = b_1_2
    else:
      varpar_v.append(b_1_2)
      varpar_e.append(e_b_1_2)
      varpar_l.append('b_1_2')
      allowed_prior_vars.append('b_1_2')
    f_1_2,e_f_1_2 = _get_param('f_1_2 = brightness factor of spot 1, star 2',
        None,0.0,None)  
    if e_f_1_2 is None:
      fixpar_d['f_1_2'] = f_1_2
    else:
      varpar_v.append(f_1_2)
      varpar_e.append(e_f_1_2)
      varpar_l.append('f_1_2')
      allowed_prior_vars.append('f_1_2')
   
  if n_spot_2 > 1:
    s_2_2,e_s_2_2 = _get_param('s_2_2 = size (degrees) of spot 2, star 2',
        None,0.001,60)  
    if e_s_2_2 is None:
      fixpar_d['s_2_2'] = s_2_2
    else:
      varpar_v.append(s_2_2)
      varpar_e.append(e_s_2_2)
      varpar_l.append('s_2_2')
      allowed_prior_vars.append('s_2_2')
    l_2_2,e_l_2_2 = _get_param('l_2_2 = longitude (degrees) of spot 2, star 2',
        None,-360,360)  
    if e_l_2_2 is None:
      fixpar_d['l_2_2'] = l_2_2
    else:
      varpar_v.append(l_2_2)
      varpar_e.append(e_l_2_2)
      varpar_l.append('l_2_2')
      allowed_prior_vars.append('l_2_2')
    b_2_2,e_b_2_2 = _get_param('b_2_2 = latitude (degrees) of spot 2, star 2',
        None,-90,90)
    if e_b_2_2 is None:
      fixpar_d['b_2_2'] = b_2_2
    else:
      varpar_v.append(b_2_2)
      varpar_e.append(e_b_2_2)
      varpar_l.append('b_2_2')
      allowed_prior_vars.append('b_2_2')
    f_2_2,e_f_2_2 = _get_param('f_2_2 = brightness factor of spot 2, star 2',
        None,0.0,None)  
    if e_f_2_2 is None:
      fixpar_d['f_2_2'] = f_2_2
    else:
      varpar_v.append(f_2_2)
      varpar_e.append(e_f_2_2)
      varpar_l.append('f_2_2')
      allowed_prior_vars.append('f_2_2')

   
  if n_spot_2 > 2:
    s_3_2,e_s_3_2 = _get_param('s_3_2 = size (degrees) of spot 3, star 2',
        None,0.001,60)  
    if e_s_3_2 is None:
      fixpar_d['s_3_2'] = s_3_2
    else:
      varpar_v.append(s_3_2)
      varpar_e.append(e_s_3_2)
      varpar_l.append('s_3_2')
      allowed_prior_vars.append('s_3_2')
    l_3_2,e_l_3_2 = _get_param('l_3_2 = longitude (degrees) of spot 3, star 2',
        None,-360,360)  
    if e_l_3_2 is None:
      fixpar_d['l_3_2'] = l_3_2
    else:
      varpar_v.append(l_3_2)
      varpar_e.append(e_l_3_2)
      varpar_l.append('l_3_2')
      allowed_prior_vars.append('l_3_2')
    b_3_2,e_b_3_2 = _get_param('b_3_2 = latitude (degrees) of spot 3, star 2',
        None,-90,90)
    if e_b_3_2 is None:
      fixpar_d['b_3_2'] = b_3_2
    else:
      varpar_v.append(b_3_2)
      varpar_e.append(e_b_3_2)
      varpar_l.append('b_3_2')
      allowed_prior_vars.append('b_3_2')
    f_3_2,e_f_3_2 = _get_param('f_3_2 = brightness factor of spot 3, star 2',
        None,0.0,None)  
    if e_f_3_2 is None:
      fixpar_d['f_3_2'] = f_3_2
    else:
      varpar_v.append(f_3_2)
      varpar_e.append(e_f_3_2)
      varpar_l.append('f_3_2')
      allowed_prior_vars.append('f_3_2')

  if n_poly==0 and n_subsets==1 :
    basis_funcs = None
  else:
    mask_fit = lcdata['flag'] >= 0
    t_obs_fit = (lcdata[mask_fit])['time']
    mag_obs_fit = (lcdata[mask_fit])['mag']
    err_obs_fit = (lcdata[mask_fit])['e_mag']
    t_pivot = np.average(t_obs_fit,weights=1.0/err_obs_fit**2)
    basis_funcs = np.vander(lcdata['time']-t_pivot,1+n_poly)
    if n_subsets>1 :
      subsets = lcdata['subset']
      for s in np.unique(subsets[subsets > 0]):
         basis_funcs = np.hstack((basis_funcs,np.transpose([subsets==s])))

  ndim = len(varpar_v)
  nwalkers,dummy = _get_param('Number of emcee walkers',
      ndim+1, 2*ndim, None, integer=True)  
  nsteps,dummy = _get_param('Number of steps in the emcee chain',
      1000, 1, None, integer=True)  
  nthreads,dummy = _get_param('Number of threads for emcee',
      1, 1, None, integer=True)  

  print('\n Now enter list of priors on parameters.\n')
  print(' In addition to parameters above (r_1, r_2, s, etc.), priors can also')
  print(' set on the following quantities:')
  print('  e = eccentricity')
  print('  om = longitude of periastron for star 1 (degrees)')
  print('  rsum = R_1/a + R_2/a')
  print('  k = R_2/R_1')
  print('  lrat = s*k^2 = luminosity ratio, L_2/L_1')

  print('\n Priors are entered using a parameter name followed by the type of ')
  print(' prior (U, G, P or S) and the parameters of the prior. ')
  print(' Prior types are as follows: ') 
  print('  U(lo_lim, hi_lim) - uniform between the limits lo_lim, hi_lim') 
  print('  G(mean, sigma) - Gaussian with specified mean and std. dev. = sigma')
#  print('  P(t_pri, e_pri) - Time of mid primary eclipse (T_0 only).')
#  print('  S(t_sec, e_sec) - Time of mid secondary eclipse (T_0 only).')
  print(' Finish list of priors with "end". ')
  print(' e.g., ')
  print('  e G 0.15 0.05')
  print('  r_2 U 0.01 0.1') 
#  print('  T_0 P 4567.234 0.001') 
#  print('  T_0 P 4569.437 0.002') 
#  print('  T_0 S 4789.331 0.001') 
  print('  end \n')

  priors = np.zeros(0, dtype={
    'names':[str('p_var'), str('p_type'),str('p_par1'),str('p_par2')],
    'formats':['S5','S1','f8','f8']})

  allowed_prior_types = ['U','G']

  prompt = " Enter prior : var U/G lo_lim/mean hi_lim/sigma : "

  while True:
    line = input(prompt)
    line = line.lstrip()
    line = (line.split('#'))[0]   # and ignore text after # on data lines
    if len(line) == 0 : continue  # Ignore blank lines/comment lines
    linedata = line.split()
    if (linedata[0]).lower() == "end": break
    if len(linedata) < 4:
      if args.tolerant: continue
      raise Exception ('Invalid entry ',line)
    p_var,p_type,p_str1,p_str2 = linedata[0:4]

    if p_var not in allowed_prior_vars:
      if args.tolerant:
        print ('No such variable',p_var)
        continue
      raise Exception ('No such variable',p_var)

    if p_type not in allowed_prior_types:
      if args.tolerant:
        print ('No such prior type',p_type)
        continue
      raise Exception ('No such prior type',p_type)

    try:
      p_par1 = np.float(p_str1)
    except:
      if args.tolerant:
        print('Failed to read floating point value ',p_str1)
        continue
      raise Exception('Failed to read floating point value ',p_str1)

    try:
      p_par2 = np.float(p_str2)
    except:
      if args.tolerant:
        print('Failed to read floating point value ',p_str2)
        continue
      raise Exception('Failed to read floating point value ',p_str2)

    
    if p_var in fixpar_d:
      print(' ### Warning - ignoring prior on fixed parameter : ',line)
      print(' ### Ignored prior on fixed parameter : ',line)
    elif p_var in priors['p_var']:
      print(' ### Warning - ignoring duplicate prior : ',line)
      print(' ### Ignored duplicate prior : ',line)
    else:
      priors = np.append(priors,
          np.array([(p_var,p_type,p_par1,p_par2)],dtype=priors.dtype))
      print('\n Including prior : ',line)
      print(line,'    #+  Prior',file=f_log)

  print('end    #+  End of priors',file=f_log)
  n_prior = len(priors)

  print('# --  End of user input block -- \n',file=f_log)
  print ('\n No. of free parameters = ',ndim)
  print ('\n No. of free parameters = ',ndim, file=f_log)
  print ('\n No. of priors = ',n_prior)
  print ('\n No. of priors = ',n_prior, file=f_log)
  t_fit = (lcdata[lcdata['flag'] >= 0])['time']
  n_fit = len(t_fit)
  n_df = n_fit + np.count_nonzero(priors['p_type'] == b'G') - ndim
  print(' Degrees of freedom (including Gaussian priors) =  ',n_df)


# Limb- and gravity- darkening instance
  ldy_ = ldy.LimbGravityDarkeningCoeffs(band)

  if args.init_only:
    print("\n Writing initial model fit to  ",args.model_file)
    print("\n Writing initial model fit to  ",args.model_file,file=f_log)
    best_pars = varpar_v

  else:
  
    # Initialize walkers so that none are out-of-bounds
    pos = []
    print("\n Initialising walkers ...")
    print("\n Initialising walkers ...",file=f_log)
    j = 0
    for i in range(nwalkers):
      lnlike_i = -np.inf
      while lnlike_i == -np.inf:
        j = j + 1
        pos_i = varpar_v + varpar_e*np.random.randn(ndim) 
        lnlike_i = _lnlike(pos_i,lcdata, basis_funcs, varpar_l, fixpar_d,
            priors,ldy_,n_spot_1, n_spot_2,args.grid_size)
      pos.append(pos_i)
    msg = " Initialised {} walkers after {} trials\n".format(nwalkers,j)
    print(msg)
    print(msg,file=f_log)

    sampler = emcee.EnsembleSampler(nwalkers, ndim, _lnlike,
        args=(lcdata, basis_funcs, varpar_l, fixpar_d, priors,ldy_,
          n_spot_1, n_spot_2, args.grid_size),threads=nthreads)
    if (args.nometer):
      sampler.run_mcmc(pos, nsteps)
    else:
      width=64
      start_time = time_()
      for i, result in enumerate(sampler.sample(pos, iterations=nsteps)):
        n = int((width+1) * float(i) / nsteps)
        delta_t = time_()-start_time
        time_incr = delta_t/(float(i+1) / nsteps)
        time_left = time_incr*(1- float(i) / nsteps)
        m, s = divmod(time_left, 60)
        h, m = divmod(m, 60)
        sys.stdout.write("\r[{}{}] {:4.2f}% - {}h:{}m:{:04.1f}s"
            .format('#'*n, ' ' * (width - n), 100*float(i)/nsteps,h,m,s))

    af = sampler.acceptance_fraction
    print('\nMedian acceptance fraction =',np.median(af))
    print('\nMedian acceptance fraction =',np.median(af),file=f_log)

    t = Table(sampler.flatchain,names=varpar_l)
    t.add_column(Column(sampler.flatlnprobability,name='loglike'))
    indices = np.mgrid[0:nwalkers,0:nsteps]
    step = indices[1].flatten()
    walker = indices[0].flatten()
    t.add_column(Column(step,name='step'))
    t.add_column(Column(walker,name='walker'))
    if (args.chain_file).find('.fits') == len(args.chain_file)-5 : 
      t.write(args.chain_file,overwrite=args.overwrite)
    else:
      t.write(args.chain_file)
   
    best_index = np.unravel_index(np.argmax(sampler.lnprobability),
        (nwalkers, nsteps))
    best_lnlike =  np.max(sampler.lnprobability)
    print('\n Best log(likelihood) = ',best_lnlike,' in walker ',best_index[0],
      ' at step ',best_index[1])
 
    best_pars = sampler.chain[best_index[0],best_index[1],:]
    for i,v in enumerate(best_pars):
      print(varpar_l[i],' : ',v)
    print("\n Writing best model fit to  ",args.model_file)
    print("\n Writing best model fit to  ",args.model_file,file=f_log)

  mag_model = _lnlike(best_pars, lcdata, basis_funcs, varpar_l, 
      fixpar_d, priors, ldy_, n_spot_1, n_spot_2, args.grid_size,
      return_fit=True)

  try:
    p = best_pars[(([i for i,s in enumerate(varpar_l) if s=='P'])[0])]
  except:
    p = fixpar_d['P']

  try:
    t0 = best_pars[(([i for i,s in enumerate(varpar_l) if s=='T_0'])[0])]
  except:
    t0 = fixpar_d['T_0']
  phase = ((lcdata['time'] - t0)/p) % 1

  try:
    sig_ext=best_pars[(([i for i,s in enumerate(varpar_l) if s=='sig_ext'])[0])]
  except:
    sig_ext = fixpar_d['sig_ext']
  print ('Assuming standard error for external noise of ',sig_ext)
  print ('Assuming standard error for external noise of ',sig_ext,file=f_log)

  t = Table(lcdata)
  t.add_column(Column(phase,name='phase'))
  t.add_column(Column(mag_model,name='model'))
  if (basis_funcs is None):
    detrend = np.zeros(n_data)
  else:
    mask_fit = lcdata['flag'] >= 0
    mag_obs_fit = (lcdata[mask_fit])['mag']
    err_obs_fit = (lcdata[mask_fit])['e_mag']
    mag_model_fit = mag_model[mask_fit]
    wt_obs_fit = 1./(err_obs_fit**2 + sig_ext**2)
    basis_funcs_fit = basis_funcs[mask_fit]
    res_obs_fit = mag_obs_fit-mag_model_fit
    c,e_c = _detrending_coeffs(basis_funcs_fit,res_obs_fit,wt_obs_fit,
        return_errors=True)
    detrend = np.dot(basis_funcs,c)
    print (c)
    print (e_c)

  mag_fit = mag_model + detrend

  s_mag = np.sqrt(lcdata['e_mag']**2 + sig_ext**2) 
  t.add_column(Column(s_mag,name='s_mag'))
  t.add_column(Column(mag_fit,name='fit'))


  if (args.model_file).find('.fits') == len(args.model_file)-5 : 
    t.write(args.model_file,overwrite=args.overwrite)
  else:
    t.write(args.model_file)

  datetime_end = datetime.today()
  print("\n End ",__file__, "{:%c}\n".format(datetime_end))
  print("\n End ",__file__, "{:%c}\n".format(datetime_end),file=f_log)

#-------------------------------
if __name__ == "__main__":
    main()



#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')
import numpy as np
import matplotlib.pyplot as plt
from astropy.table import Table, Column

#------------------------------------------------------------------------------

# Handy class for printing floats...
class prettyfloat(float):
  def __repr__(self):
    return "%0.6f" % self

#------------------------------------------------------------------------------
# Generate plots

model = Table.read('model.csv')
model = Table(model, masked=True)

mask = model['flag'] < 0 

fig=plt.figure(1,figsize=(12,8))

ph_obs = np.extract(mask == False,model['phase']) 
m_obs = np.extract(mask == False,model['mag'])
m_fit = np.extract(mask == False,model['fit'])
m_res = m_obs - m_fit
print ('RMS residual = {0:0.3f}'.format(1000*np.std(m_res)))
ph_plt = np.array(model['phase']) 
f_plt = np.array(model['fit'])
i_sort = np.argsort(ph_plt)
ph_plt = ph_plt[i_sort]
f_plt  = f_plt[i_sort]
fontsize=12

plt.subplot(321)
plt.scatter(ph_obs,m_obs,color='darkgreen',marker='x',s=3)
plt.scatter(ph_obs-1,m_obs,color='darkgreen',marker='x',s=3)
plt.plot(ph_plt,f_plt,color='darkblue')
plt.plot(ph_plt-1,f_plt,color='darkblue')
# plt.scatter(ph_obs,m_res-0.01,color='darkgreen',marker='x',s=3)
# plt.scatter(ph_obs-1,m_res-0.01,color='darkgreen',marker='x',s=3)
# plt.plot([-0.08,0.08],[-0.01,-0.01],':',color='darkblue')
plt.ylabel("Kp [mag]",fontsize=fontsize)
plt.xlim([-0.08,0.08])
plt.ylim([0.12,-0.02])

plt.subplot(322)
plt.scatter(ph_obs,m_obs,color='darkgreen',marker='x',s=3)
# plt.scatter(ph_obs,m_res-0.01,color='darkgreen',marker='x',s=3)
plt.plot(ph_plt,f_plt,color='darkblue')
# plt.plot([ 0.42,0.58],[-0.01,-0.01],':',color='darkblue')
plt.xlim([ 0.42,0.58])
plt.ylim([0.12,-0.02])

plt.subplot(312)
plt.scatter(ph_obs,m_obs,color='darkgreen',marker='x',s=3)
plt.scatter(ph_obs-1,m_obs,color='darkgreen',marker='x',s=3)
plt.plot(ph_plt,f_plt,color='darkblue')
plt.plot(ph_plt-1,f_plt,color='darkblue')
plt.ylabel("Kp [mag]",fontsize=fontsize)
plt.xlim([-0.35,0.85])
plt.ylim([0.12,-0.01])


plt.subplot(313)
plt.scatter(ph_obs,m_res,color='darkgreen',marker='x',s=3)
plt.scatter(ph_obs-1,m_res,color='darkgreen',marker='x',s=3)
plt.plot([-0.35,0.85],[0,0],color='darkblue')
plt.ylabel("O-C [mag]",fontsize=fontsize)
plt.xlabel("Phase",fontsize=fontsize)
plt.xlim([-0.35,0.85])
plt.ylim([0.0035,-0.0035])



if args.eps:
  fig.savefig("lcfit.eps")
else:
  plt.show()






#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file",action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import matplotlib.pyplot as plt
import ellc

M_1 = 0.43  
M_2 = 0.17
R_1 = 0.0148
R_2 = 0.0214
period = 39.1/60/24
incl = 90.0
vsini_1 = 1148.8
vsini_2 = 601.3
bfac_1 = 0.36
bfac_2 = 1.10
rotfac_1 = vsini_1/(50.57877*R_1/period)
rotfac_2 = vsini_2/(50.57877*R_2/period)
ld_1 = 'lin'
ld_2 = 'lin'
ldc_1 = 0.4
ldc_2 = 0.6
gdc_1 = 0.4
gdc_2 = 0.4

sbratio = 16485/10000  # = T_2/T_1
t_0 = 0.5*period  # .. to get phases consistent with Groot's Fig. 1

a = 4.20944009361 * period**(2./3.) * (M_1+M_2)**(1./3.)
r_1 = R_1/a
r_2 = R_2/a


t1 = np.arange(-0.2,0.8 , 0.001)*period
pro= ellc.lc(t1, radius_1=r_1, radius_2=r_2, sbratio=sbratio, incl=incl,
     q=M_2/M_1, a=a, period=period,t_zero=t_0,bfac_1=bfac_1, bfac_2=bfac_2, 
     ld_1=ld_1,ld_2=ld_2,ldc_1=ldc_1,ldc_2=ldc_2,gdc_1=gdc_1, gdc_2=gdc_2,
     rotfac_1=rotfac_1, rotfac_2=rotfac_2,shape_1='roche',shape_2='roche')
ret= ellc.lc(t1, radius_1=r_1, radius_2=r_2, sbratio=sbratio, incl=incl,
     q=M_2/M_1, a=a, period=period,t_zero=t_0,bfac_1=bfac_1, bfac_2=bfac_2, 
     ld_1=ld_1,ld_2=ld_2,ldc_1=ldc_1,ldc_2=ldc_2,gdc_1=gdc_1, gdc_2=gdc_2,
     rotfac_1=-rotfac_1, rotfac_2=-rotfac_2,shape_1='roche',shape_2='roche')
incl = 89.0
l30 =ellc.lc(t1, radius_1=r_1, radius_2=r_2, sbratio=sbratio, incl=incl,
     q=M_2/M_1, a=a, period=period,t_zero=t_0,bfac_1=bfac_1, bfac_2=bfac_2, 
     ld_1=ld_1,ld_2=ld_2,ldc_1=ldc_1,ldc_2=ldc_2,gdc_1=gdc_1, gdc_2=gdc_2,
     vsini_1=vsini_1, vsini_2=vsini_2, lambda_1=30, lambda_2=30) 
l210=ellc.lc(t1, radius_1=r_1, radius_2=r_2, sbratio=sbratio, incl=incl,
     q=M_2/M_1, a=a, period=period,t_zero=t_0,bfac_1=bfac_1, bfac_2=bfac_2, 
     ld_1=ld_1,ld_2=ld_2,ldc_1=ldc_1,ldc_2=ldc_2,gdc_1=gdc_1, gdc_2=gdc_2,
     vsini_1=vsini_1, vsini_2=vsini_2, lambda_1=210, lambda_2=210) 

fig=plt.figure(1,figsize=(6,5))
plt.subplot(211)
plt.xlim([-0.1,0.6])
plt.ylim([0.55,1.05])
plt.ylabel('Flux')
plt.plot(t1/period,pro,color='darkblue')
plt.plot(t1/period,l30,color='darkgreen',linestyle='--')

plt.subplot(223)
plt.locator_params(axis = 'x', nbins = 4)
plt.xlim([-0.03,0.03])
plt.ylim([-800,1200])
plt.ylabel('R-M effect [PPM]')
plt.xlabel('Phase')
plt.plot(t1/period, 0.5*(pro-ret)*1e6,color='darkblue')
plt.plot(t1/period, 0.5*(l30-l210)*1e6,color='darkgreen',linestyle='--')

plt.subplot(224)
plt.xlim([ 0.47,0.53])
plt.locator_params(axis = 'x', nbins = 4)
plt.ylabel('Flux difference [PPM]')
plt.ylim([-200,200])
plt.xlabel('Phase')
plt.plot(t1/period  , 0.5*(pro-ret)*1e6,color='darkblue')
plt.plot(t1/period, 0.5*(l30-l210)*1e6,color='darkgreen',linestyle='--')

plt.tight_layout()

if args.eps:
  fig.savefig("PhotRM.eps",dpi=400)
else:
  plt.show()




#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file",action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')
import ellc
import batman
import numpy as np
import matplotlib.pyplot as plt

lc = np.loadtxt('kplr007975824.dat',
     dtype={'names':('Phase','Flux'), 'formats': ('f8','f8')})

# Reversing the roles of star 1 and 2 cf. Bloemen et al.
period = 0.40375026
K_2 = 164.0
M_1 = 0.59
M_2 = 0.47
incl = 87.14
a_2 = 0.019771142 * K_2 * period #  * np.sqrt(1 - ecc**2)/np.sin(incl*np.pi/180)
q = M_2/M_1
a = (1+q)*a_2
r_1 = 0.013/a
r_2 = 0.214/a
sbratio = 2.86 
gdc_2 = 0.448  # beta_K
ld_1='claret'
ldc_1 = [0.832, -0.681, 0.621, -0.239]
ld_2 = 'claret'
ldc_2 = [0.818, -0.908, 0.755, -0.252] 

bfac_2 = 1.30

flux_r = ellc.lc(lc['Phase']*period, period=period, t_zero=0,
    radius_1=r_1,  radius_2=r_2, incl=incl, \
    sbratio=sbratio, ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, \
    gdc_2=gdc_2, bfac_2=bfac_2, a=a, q=q, shape_1='roche',shape_2='roche')

# Roche potential, exact local gravity calculation
flux_e = ellc.lc(lc['Phase']*period, period=period, t_zero=0,
    radius_1=r_1,  radius_2=r_2, incl=incl, \
    sbratio=sbratio, ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, \
    gdc_2=gdc_2, bfac_2=bfac_2, a=a, q=q, shape_1='roche',shape_2='roche',
    exact_grav=True)
print("Max. difference between approx. and exact gravity" +
    " calculation = {0:.1f} ppm".format(1e6*np.max(abs(flux_r-flux_e))))

flux_p = ellc.lc(lc['Phase']*period, period=period, t_zero=0,
    radius_1=r_1,  radius_2=r_2, incl=incl, \
    sbratio=sbratio, ld_1=ld_1, ldc_1=ldc_1, ld_2=ld_2, ldc_2=ldc_2, \
    gdc_2=gdc_2, bfac_2=bfac_2, a=a, q=q, shape_1='roche',shape_2='poly1p5')


flux_r = flux_r/np.median(flux_r/lc['Flux'])
flux_p = flux_p/np.median(flux_p/lc['Flux'])

fig=plt.figure(1,figsize=(12,5))
plt.subplot(211)
plt.xlim([-0.4,0.9])
plt.ylim([0.9945,1.0035])
plt.scatter(lc['Phase']   ,lc['Flux'],color='k',s=1)
plt.scatter(lc['Phase']-1,lc['Flux'],color='k',s=1)
plt.plot(lc['Phase']   ,flux_r,color='darkred')
plt.plot(lc['Phase']-1,flux_r,color='darkred')
plt.plot(lc['Phase']   ,flux_p,'--',color='darkblue')
plt.plot(lc['Phase']-1,flux_p,'--',color='darkblue')
plt.xlabel("Phase")
plt.ylabel("Flux")

plt.subplot(223)
plt.xlim([-0.05,0.05])
plt.ylim([0.996,1.001])
plt.scatter(lc['Phase']   ,lc['Flux'],color='k',s=1)
plt.scatter(lc['Phase']-1,lc['Flux'],color='k',s=1)
plt.plot(lc['Phase']   ,flux_r,color='darkred')
plt.plot(lc['Phase']-1,flux_r,color='darkred')
plt.plot(lc['Phase']   ,flux_p,'--',color='darkblue')
plt.plot(lc['Phase']-1,flux_p,'--',color='darkblue')
plt.xlabel("Phase")
plt.ylabel("Flux")
plt.locator_params(axis = 'x', nbins = 4)

plt.subplot(224)
plt.xlim([0.45,0.55])
plt.ylim([0.994,1.00])
plt.scatter(lc['Phase']   ,lc['Flux'],color='k',s=1)
plt.plot(lc['Phase']   ,flux_r,color='darkred')
plt.plot(lc['Phase']   ,flux_p,'--',color='darkblue')
plt.xlabel("Phase")
plt.ylabel("Flux")
plt.locator_params(axis = 'x', nbins = 4)

plt.tight_layout()
if args.eps:
  fig.savefig("KPD1946+4340.eps")
else:
  plt.show()



#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
from ellc import ldy,lc

import matplotlib.pyplot as plt


lc_dat = np.loadtxt("NightfallCurve.dat")

phase = lc_dat[:,1]
flux_q = 10**(-0.4*lc_dat[:,6])  # 
flux_q = flux_q /np.min(flux_q)

# Star 1 = "Secondary" to that t_zero = 0
sbratio = 1
r_2 = 0.166822
r_1 = 0.007043 
incl = 80.0
q = 0.5
shape_1 = 'sphere'
ldy_ = ldy.LimbGravityDarkeningCoeffs('I')
a1,a2,a3,a4,y = ldy_(20000,4.0,0.0)
# y = 0
print('y =',y)
ld_2 = 'claret'
ldc_2 = [a1,a2,a3,a4]
ecc = 0.15
om  = (180+120) * np.pi/180.
f_c = np.sqrt(ecc)*np.cos(om)
f_s = np.sqrt(ecc)*np.sin(om)
shape_2 = 'roche'
lc_1 = lc(phase,t_zero=0, q=q, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_2=ld_2, ldc_2=ldc_2, gdc_2=y,
    f_c=f_c, f_s=f_s,
    shape_1=shape_1,shape_2=shape_2,exact_grav=True) 
lc_2 = lc(phase,t_zero=0, q=q, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_2=ld_2, ldc_2=ldc_2, gdc_2=y,
    f_c=f_c, f_s=f_s,
    shape_1=shape_1,shape_2=shape_2,exact_grav=False) 
shape_2 = 'roche_v'
lc_3 = lc(phase,t_zero=0, q=q, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_2=ld_2, ldc_2=ldc_2, gdc_2=y,
    f_c=f_c, f_s=f_s,
    shape_1=shape_1,shape_2=shape_2,exact_grav=True) 
lc_4 = lc(phase,t_zero=0, q=q, 
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,  
    ld_2=ld_2, ldc_2=ldc_2, gdc_2=y,
    f_c=f_c, f_s=f_s,
    shape_1=shape_1,shape_2=shape_2,exact_grav=False) 

flux_q = flux_q/np.median(flux_q)
lc_1 = lc_1/np.median(lc_1)
lc_2 = lc_2/np.median(lc_2)
lc_3 = lc_3/np.median(lc_3)
lc_4 = lc_4/np.median(lc_4)


fig=plt.figure(1,figsize=(8,6))
line_q,=plt.plot(phase,flux_q,color='k',label='Nightfall')
line_1,=plt.plot(phase,lc_1,color='g',linestyle='--',label='ellc: exact, roche')
line_2,=plt.plot(phase,lc_2,color='b',linestyle=':',label='ellc: poly, roche')
line_3,=plt.plot(phase,lc_3,color='r',linestyle='-.',label='ellc: exact, roche_v')
line_4,=plt.plot(phase,lc_4,color='c',linestyle='-',label='ellc: poly, roche_v')
plt.xlim([-0.25,0.75])
plt.ylim([0.95,1.05])
plt.xlabel("Phase")
plt.ylabel("Flux")
plt.legend(handles=[line_1,line_2,line_3,line_4,line_q],loc='upper right')

plt.tight_layout()

if args.eps:
  fig.savefig("Ellipsoidal.eps")
else:
  plt.show()



#!/usr/bin/env python
from __future__ import (absolute_import, division, print_function,
                            unicode_literals)
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import ellc
import matplotlib.pyplot as plt

# Output from arome-1.0.0/examples/f77multiple.f
#
arome_dat = np.loadtxt("arome.dat")

phase  = arome_dat[:,0]
f      = arome_dat[:,4]
v_mean = arome_dat[:,10]

# For testing purposes, calculate R-M effect twice using both star 1 and star 2 in
# the role of the planet.
# For calculation of orbital velocity, assume R_* = 1R_sun

# !/* planet orbital parameters */
# sma = 4.0d0              !/* stellar radii */
# inc = 86.0d0*pi/180.0d0  !/* radian */
a = 4.0
r_star = 1/a
incl = 86.0

# !/* spin-orbit angle */
# lambda = 30.0d0*pi/180.0d0 !/* radian */
lambda_star = 30.0

# !/* limb-darkening */
# c1 = 0.701d0
# c2 = 0.149d0
# c3 = 0.277d0
# c4 =-0.297d0
ld = 'claret'
ldc = [0.701, 0.149, 0.277, -0.297]

# !/* line profile */
# Vsini  = 15.0d0  !/* Vsini */
# Rp     =  0.1d0  !/* radius of the planet */
vsini = 15.0
r_planet = 0.1*r_star

q  = 0.0001  # Nominal mass ratio
# For calculation of period, assume M_* = 1M_sun
period = (a / 4.20944009361)**(3./2.)

t = np.arange(0.18,0.32,0.0005)
phase_1 = (t-0.25)/period + 0.25
f_1 = ellc.lc(t, radius_1 = r_star, radius_2 = r_planet, sbratio = 1e-9, 
    incl=incl, t_zero=0.25, period=period, a=a, q=q, ldc_1=ldc,
    lambda_1=lambda_star, vsini_1=vsini, ld_1='claret',shape_1='sphere')
rv_1,dum = ellc.rv(t, radius_1 = r_star, radius_2 = r_planet, sbratio = 1e-9, 
      incl=incl, t_zero=0.25, period=period, a=a, q=q, ldc_1=ldc,
      lambda_1=lambda_star, vsini_1=vsini, ld_1='claret',shape_1='sphere',
      flux_weighted=True)
rv_orb_1,dum = ellc.rv(t, radius_1 = r_star, radius_2 = r_planet, sbratio = 1e-9, 
      incl=incl, t_zero=0.25, period=period, a=a, q=q, ldc_1=ldc,
      lambda_1=lambda_star, vsini_1=vsini, ld_1='claret',shape_1='sphere',
      flux_weighted=False)
rm_1 = rv_1 - rv_orb_1

# Light travel time correction
solar_radius = 6.957e8
c  = 2.99792458e8
ltt = a*solar_radius/c/86400.0

phase_2 = (t-0.25)/period + 0.25
t_zero = 0.25 + 0.5*period + ltt
f_2 = ellc.lc(t, radius_2 = r_star, radius_1 = r_planet, sbratio = 1e9, 
    incl=incl, t_zero=t_zero, period=period, a=a, q=1/q, ldc_2=ldc,
    lambda_2=lambda_star, vsini_2=vsini, ld_2='claret',shape_2='sphere')
dummy,rv_2 = ellc.rv(t, radius_2 = r_star, radius_1 = r_planet, sbratio = 1e9, 
      incl=incl, t_zero=t_zero, period=period, a=a, q=1/q, ldc_2=ldc,
      lambda_2=lambda_star, vsini_2=vsini, ld_2='claret',shape_2='sphere',
      flux_weighted=True)
dummy,rv_orb_2 = ellc.rv(t, radius_2 = r_star, radius_1 = r_planet, sbratio = 1e9, 
      incl=incl, t_zero=t_zero, period=period, a=a, q=1/q, ldc_2=ldc,
      lambda_2=lambda_star, vsini_2=vsini, ld_2='claret',shape_2='sphere',
      flux_weighted=False)
rm_2 = rv_2 - rv_orb_2

fig=plt.figure(1,figsize=(8,8))

plt.subplot(211)
plt.scatter(phase,1-f,color='k',label='arome')
plt.plot(phase_1,f_1,color='b')
plt.plot(phase_2,f_2,color='g')
plt.xlim([ 0.18,0.32])
plt.xlabel("Phase")
plt.ylabel("Flux")

plt.subplot(212)
plt.scatter(phase,v_mean,color='k',label='arome')
plt.plot(phase_1,rm_1,color='b')
plt.plot(phase_2,rm_2,color='g')
plt.xlim([ 0.18,0.32])
plt.xlabel("Phase")
plt.ylabel("V_RM [km/s]")

plt.tight_layout()

if args.eps:
  fig.savefig("arome.eps")
else:
  plt.show()



#!/usr/bin/env python
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="generate .eps file",action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import numpy as np
import ellc
import matplotlib.pyplot as plt

lc_dat = np.loadtxt("NightfallCurve.dat")

phase = lc_dat[:,1]
flux = 10**(-0.4*lc_dat[:,4])  # V-band
rv_1 = lc_dat[:,14]
rv_2 = lc_dat[:,15]

# Note that star 1 is the star labelled Secondary in nightfall
t_zero = 0.0
period = 4.0
sbratio = 1.4
r_1 = 0.09992 
r_2 = 0.09726
incl = 89.99
a = 16.826
q = 1/0.8
rotfac_1 = 1
rotfac_2 = 15
shape_1 = 'roche'
shape_2 = 'roche'
gdc_1 = 1
gdc_2 = 1
ld_1 = 'quad'
ldc_1 = [0.283,  0.385] # V-band coeffs from nightfall (using -Dv flag)
ld_2 = 'quad'
ldc_2 = [ 0.417, 0.285] # V-band coeffs from nightfall (using -Dv flag)
ecc = 0  # e>0 does not work in nightfall!
om  = 90
f_c = np.sqrt(ecc)*np.cos(om*np.pi/180.)
f_s = np.sqrt(ecc)*np.sin(om*np.pi/180.)

t = phase*period
lc = ellc.lc(t,t_zero=t_zero, period=period, a=a, q=q,
    ld_1=ld_1,ldc_1=ldc_1,ld_2=ld_2,ldc_2=ldc_2,
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio, 
    rotfac_1=rotfac_1, rotfac_2=rotfac_2, gdc_1=gdc_1, gdc_2=gdc_2,
    shape_1=shape_1,shape_2=shape_2, grid_1='default',grid_2='default') 
rv_ellc_2,rv_ellc_1 = ellc.rv(t,t_zero=t_zero, period=period, a=a, q=q,
    radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio, 
    rotfac_1=rotfac_1, rotfac_2=rotfac_2, gdc_1=gdc_1, gdc_2=gdc_2,
    shape_1=shape_1,shape_2=shape_2, grid_1='default',grid_2='default') 

fontsize=9
fig=plt.figure(1,figsize=(8,4))
fig=plt.figure(1)
plt.subplot(211)
plt.xlim([-0.25,0.75])
plt.ylim([0.4,1.1])
plt.plot(phase,flux/np.max(flux),color='darkblue',linestyle='--')
plt.plot(phase,lc/np.max(lc),color='darkgreen',linestyle=':')
plt.xlabel("Time [d]",fontsize=fontsize)
plt.ylabel("Flux",fontsize=fontsize)
plt.tick_params(axis='both', labelsize=fontsize)

plt.subplot(223)
plt.xlim([-0.05,0.05])
plt.ylim([-50,50])
plt.plot(phase,rv_1,color='darkblue',linestyle='--')
plt.plot(phase,rv_2,color='darkgreen',linestyle='--')
plt.plot(phase,rv_ellc_1,color='darkblue',linestyle=':' )
plt.plot(phase,rv_ellc_2,color='darkgreen',linestyle=':')
plt.locator_params(axis = 'x', nbins = 4)
plt.tick_params(axis='both', labelsize=fontsize)
plt.xlabel("Time [d]",fontsize=fontsize)
plt.ylabel("Radial velocity [km/s]",fontsize=fontsize)
plt.tick_params(axis='both', labelsize=fontsize)

plt.subplot(224)
plt.xlim([0.45,0.55])
plt.ylim([-400,400])
plt.plot(phase,rv_1,color='darkblue',linestyle='--')
plt.plot(phase,rv_2,color='darkgreen',linestyle='--')
plt.plot(phase,rv_ellc_1,color='darkblue',linestyle=':' )
plt.locator_params(axis = 'x', nbins = 4)
plt.tick_params(axis='both', labelsize=fontsize)
plt.ylabel("Radial velocity [km/s]",fontsize=fontsize)
plt.xlabel("Time [d]",fontsize=fontsize)

plt.tight_layout()
if args.eps:
  fig.savefig("DoublePartial.eps")
else:
  plt.show()




"""
  ellc
  ----

  Generate light curves or radial velocity curves for a binary star system 
  with the ellc light curve model [1].

  Routines in this module:

  lc(t_obs, radius_1, radius_2, sbratio, incl, ... ) 

  rv(t_obs, radius_1, radius_2, sbratio, incl, t_zero, period, a, q, ...)

  fluxes(t_obs, radius_1, radius_2, sbratio, incl, ... ) 

  tmin(t_obs, radius_1, radius_2, sbratio, incl, ... ) 

  Sub-packages in this module

  ldy
   - ldy.list_bands()
   - ldy.LimbGravityDarkeningCoeffs(band)


  Please cite Maxted (2016) if you published results based on this software.

  References
  ----------
  .. [1] Maxted, P.F.L. 2016. A fast, flexible light curve model for detached
      eclipsing binary stars and transiting exoplanets. A&A VOLUME, ARTICLE.

"""

try:
      from .version import __version__
except:
      __version__ = ''


try:
      from ellc.lc import lc
except:
      pass

try:
      from ellc.rv import rv
except:
      pass

try:
      from ellc.tmin import tmin
except:
      pass

try:
      from ellc.fluxes import fluxes
except:
      pass

try:
      from ellc.ldy import ldy
except:
      pass




#!/usr/bin/env python
import argparse

parser = argparse.ArgumentParser()
parser.add_argument("--eps", help="genereate .eps file", action="store_true")
args = parser.parse_args()

if args.eps:
  import matplotlib
  matplotlib.use('Agg')

import ellc
import numpy as np
import matplotlib.pyplot as plt


t = np.linspace(-0.05, 0.05, 1000)

r_1 = 0.15
r_2 = 0.02
incl = 89.5
ldc_1 = [0.1, 0.3]
ld_1 = 'quad'
sbratio = 0 
e = 0.1
om = 60
f_s = np.sqrt(e)*np.sin(om*np.pi/180.)
f_c = np.sqrt(e)*np.cos(om*np.pi/180.)
q = 0.001

flux_0 = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='sphere',
    grid_1='default',grid_2='default', f_s=f_s, f_c=f_c)

flux_2 = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='love',hf_2=2.0,q=q,
    grid_1='default',grid_2='default', f_s=f_s, f_c=f_c)

flux_4 = ellc.lc(t,radius_1=r_1, radius_2=r_2,incl=incl,sbratio=sbratio,
    ld_1=ld_1, ldc_1=ldc_1,shape_1='sphere',shape_2='love',hf_2=4.0,q=q,
    grid_1='default',grid_2='default', f_s=f_s, f_c=f_c)

fig=plt.figure(1,figsize=(6,8))
plt.subplot(211)
plt.xlim([-0.05,0.05])
plt.ylim([0.975,1.002])
line_0, = plt.plot(t, flux_0, 'k-',label='sphere')
line_2, = plt.plot(t, flux_2, 'c--',label='h_f = 2')
line_4, = plt.plot(t, flux_4, 'g:', label='h_f = 4')
plt.ylabel("Flux",fontsize=16)
plt.legend(handles=[line_0,line_2,line_4],loc='lower left')

plt.subplot(212)
plt.xlim([-0.05,0.05])
line_2, = plt.plot(t, 1e6*(flux_2-flux_0), 'c--', label='h_f = 2')
line_4, = plt.plot(t, 1e6*(flux_4-flux_0), 'g:', label='h_f = 4')
plt.ylabel("Flux difference [ppm]",fontsize=16)
plt.legend(handles=[line_2,line_4],loc='upper left')

plt.tight_layout()
if args.eps:
  fig.savefig("love.eps")
else:
  plt.show()



# This file is part of the ellc binary star model
# Copyright (C) 2017 Pierre Maxted
# 
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import (absolute_import, division, print_function,
                        unicode_literals)
import numpy as np

from ellc import ellc_f

def lc(t_obs, radius_1, radius_2, sbratio, incl, 
       light_3 = 0, 
       t_zero = 0, period = 1,
       a = None,
       q = 1,
       f_c = None, f_s = None,
       ldc_1 = None, ldc_2 = None,
       gdc_1 = None, gdc_2 = None,
       didt = None,
       domdt = None,
       rotfac_1 = 1, rotfac_2 = 1,
       hf_1 = 1.5, hf_2 = 1.5,
       bfac_1 = None, bfac_2 = None,
       heat_1 = None, heat_2 = None, 
       lambda_1 = None, lambda_2 = None,
       vsini_1 = None, vsini_2 = None,
       t_exp=None, n_int=None, 
       grid_1='default', grid_2='default',
       ld_1=None, ld_2=None,
       shape_1='sphere', shape_2='sphere',
       spots_1=None, spots_2=None, 
       exact_grav=False, verbose=1):
  """
  Calculate the light curve of a binary star

  This function calculates the light curve of a binary star using the ellc
  binary star model [1].

  Parameters
  ----------
  t_obs : array_like
      Times or phases of observation. The units and time system used must be
      consistent with t_zero and period.

  radius_1 : float
      Radius of star 1 in units of the semi-major axis of the binary.
      The radius is defined to be the same as a sphere with the same volume as
      the ellipsoid used to approximate the shape of the star.
      Set radius_1=1 to fix radius at limiting radius in the Roche potential.

  radius_2 : float
      Radius of star 2 in units of the semi-major axis of the binary.
      The radius is defined to be the same as a sphere with the same volume as
      the ellipsoid used to approximate the shape of the star.
      Set radius_2=1 to fix radius at limiting radius in the Roche potential.

  sbratio : float
      Surface brightness ratio, S_2/S_1 

  incl : float
      Inclination in degrees.

  light_3 : float, optional
      Third light contribution relative to total flux from both stars at 
      time t_zero excluding eclipse effects.

  t_zero : float, optional
      Time (or phase) of mid-eclipse for star 1 by star 2.
      The units and time system must be consistent with the values in t_obs.
      Default is 0.

  period : float, optional
      Orbital period of the binary or (for phased data) 1.
      For binary systes with apsidal motion, this is the anomalistic period.
      If light-time effect or Doppler boosting is to be calculated correctly
      then the units of period must be days, otherwise arbitrary units can be
      used provided that these are consistent with t_zero and t_obs.
      Default is 1.

  a : {None, float}, optional
      Semi-major axis in solar radii for calculation of light travel time
      and Doppler boosting.

  q : float, optional
      Mass ratio m_2/m_1.
      Default is 1.

  f_c : {None, float},  optional
      For eccentric orbits with eccentricity e and longitude of periastron w,
      f_c = sqrt(e).cos(w)
      If not None, then f_s must also be specified.
      Default is None.

  f_s : {None, float},  optional
      For eccentric orbits with eccentricity e and longitude of periastron w,
      f_s = sqrt(e).sin(w)
      If not None, then f_c must also be specified.
      Default is None.

  ldc_1 : {None, float, or array_like}, optional
      Limb darkening coefficients for star 1.
      Number of elements must match the selected limb-darkening law.
      For the option ld_1='mugrid', ldc_1 is a grid of specific
      intensity values on a uniform grid from mu=0 (first element) to mu=1
      (last element). The specific intensies are assumed to be normalised,
      i.e., ldc_1[-1] = 1, but this is not checked.
      Default is None

  ldc_2 : {None, float, or array_like}, optional
      Limb darkening coefficients for star 2.
      Number of elements must match the selected limb-darkening law.
      For the option ld_2='mugrid', ldc_2 is a grid of specific
      intensity values on a uniform grid from mu=0 (first element) to mu=1
      (last element). The specific intensies are assumed to be normalised,
      i.e., ldc_2[-1] = 1, but this is not checked.
      Default is None

  gdc_1 : {None, float},  optional
      Gravity darkening exponent for star 1.

  gdc_2 : {None, float},  optional
      Gravity darkening exponent for star 2.

  didt : {None, float},  optional
      Rate of change of inclination [degrees/anomalistic period]
      Default is None.

  domdt : {None, float},  optional
      Apsidal motion rate  [degrees/anomalistic period].
      Default is None.

  rotfac_1 : float, optional
      Asynchronous rotation factor for star 1, F_1
      Default is 1.

  rotfac_2 : float, optional
      Asynchronous rotation factor for star 2, F_2
      Default is 1.

  hf_1 : float, optional
      h_f, fluid second Love number for radial displacement, for star 1.
      Only used if shape_1 = 'love'
      Default is 1.5.

  hf_2 : float, optional
      h_f, fluid second Love number for radial displacement, for star 2.
      Only used if shape_2 = 'love'
      Default is 1.5.


      N.B. Doppler boosting is not calculated is parameter a is None
      Default is None.
      
  bfac_2 : {None, float}, optional
      Doppler boosting factor, star 2
      N.B. Doppler boosting is not calculated is parameter a is None
      Default is None.
      
  heat_1 : {None, scalar, 3-tuple of floats}, optional
      If scalar, coefficient of simplified reflection model - see Notes.
      If 3-tuple, parameters of heating+reflection model, [H_0, H_1, u_H]
      H_0 is the coefficient, H_1 is the exponent and u_H is the linear
      limb-darkening coefficient.
      Default is None.
  
  heat_2 : {None, scalar, 3-tuple of floats}, optional
      If scalar, coefficient of simplified reflection model - see Notes.
      If 3-tuple, parameters of heating+reflection model, [H_0, H_1, u_H]
      H_0 is the coefficient, H_1 is the exponent and u_H is the linear
      limb-darkening coefficient.
      Default is None.
  
  lambda_1 : {None, float},  optional
       Sky-projected angle between orbital and rotation axes, star 1 [degrees]
       N.B. lambda_1 is only used if shape_1='sphere'
       Default is None.
  
  lambda_2 : {None, float},  optional
       Sky-projected angle between orbital and rotation axes, star 2 [degrees]
       N.B. lambda_2 is only used if shape_2='sphere'
       Default is None.

  vsini_1  : {None, float}, optional
      V_rot.sini for calculation of R-M effect for star 1 [km/s]
      See notes below.
      Default is None.

  vsini_2  : {None, float}, optional
      V_rot.sini for calculation of R-M effect for star 2 [km/s]
      See notes below.
      Default is None.

  t_exp : {None, float, array_like}, optional
      Exposure time in the same units as t_obs and, if array_like, with the
      same number of elements.
      Default is None.

  n_int : {None, int, array_like}
      Number of integration points used to account for finite exposure time.
      Set n_int or elements of n_int to 1 to make no correction for finite
      integration time.
      If array_like then set elements to 0 to use linear interpolation of the
      other values in the light curve to estimate the light curve at the
      corresponding values of t_obs. 
      Default is None.

  grid_1 : {"very_sparse", "sparse", "default", "fine", "very_fine"}, optional
      Grid size used to calculate the flux from star 1.
      Default is "default"

  grid_2 : {"very_sparse", "sparse", "default", "fine", "very_fine"}, optional
      Grid size used to calculate the flux from star 2.
      Default is "default"

  ld_1 : {None, "lin", "quad", "sing", "claret", "log", "sqrt", "exp",
          "power-2", "mugrid"} 
   Limb darkening law for star 1
   The "power-2" limb darkening law is taken from  Morello et al., 
   2017AJ....154..111M:
    I_lambda(mu)/I_lambda(1) = 1 - ldc_1[0] * (1 - mu**ldc_1[1])
   For the option "mugrid", ldc_1 must be an array of specific intensity
   values as a function of mu=cos(theta), where theta is the angle between the
   line of sight and the normal to a given point of the stellar surface. The
   value of mu corresponding to element i of ldc_1 is i/(len(ldc_1)-1),  where
   i=0,1, ... len(ldc_1)-1.  
   Default is None

  ld_2 : {None, "lin", "quad", "sing", "claret", "log", "sqrt", "exp",
          "power-2", "mugrid"} 
   Limb darkening law for star 2.
   See ld_1 for description of the power-2 and mugrid options.
   Default is None

  shape_1 : {"roche", "roche_v", "sphere", "poly1p5", "poly3p0", "love"}
      Model used to calculate the shape of star 1 - see Notes. 
      Default is "sphere".

  shape_2 : {"roche", "roche_v", "sphere", "poly1p5", "poly3p0", "love"}
      Model used to calculate the shape of star 2 - see Notes. 
      Default is "sphere".

  spots_1 : (4, n_spots_1) array_like
   Parameters of the spots on star 1. For each spot the parameters, in order,
   are longitude, latitude, size and brightness factor. All three angles are
   in degrees.

  spots_2 : (4, n_spots_2) array_like
   Parameters of the spots on star 2. For each spot the parameters, in order,
   are longitude, latitude, size and brightness factor. All three angles are
   in degrees.

  exact_grav : {True|False}
    Use point-by-point calculation of local surface gravity for calculation of
    gravity darkening is True, otherwise use (much faster) approximation based
    on functional form fit to local gravity at 4 points on the star.

  Returns
  -------
  flux : ndarray
      Flux in arbitrary units.

  Notes
  -----

   The asynchronous rotation factors rotfac_1 and rotfac_2 are used to
  calculate the shapes of the stars (unless spherical stars are specified).
  These rotation factors are relative to the actual synchronous rotation rate
  (rotation period = orbital period) not pseudo-synchronous rotation rate.

   The effect of the spot on the light curve is calculated using the
  algorithm by Eker [2] for circular spots on a spherical star with
  quadratic limb darkening. If the limb-darkening law used for the main
  calculation is not linear or quadratic then the coefficients of the
  limb-darkening law used for the calculation of the effects of the spots
  are set so that the intensity distribution matches at mu = 0, 0.5 and 1.

   N.B. The effect of each spot on the light curve is additive so overlapping
  spots can result in non-physical negative fluxes for some regions of the
  star.
 
   For the calculation of the star shape, the rotation and orbital angular
  momentum vectors are assumed parallel. The shape of each star is
  approximated by a triaxial ellipsoid with semi-major axes (A,B,C) and
  offset towards companion, D, from the centre-of-mass of the star towards
  the companion star. For the option "roche" the definition of the Roche
  potential from Wilson [3] is used and the values of A, B, C, D are set to
  that intersection points of the triaxial ellipsoid with x-, y- and z-axes
  lie on an equipotential surface. For the options "poly1p5" and "poly3p0"
  the star is assumed to behave as a polytrope with index n=1.5 or n=3,
  respectively. The tidal and rotational distortion of the polytrope are 
  assumed to be independent. The tidal distortion for polytropes is from
  Chandrasekhar [4] and the rotational distortion is calculated by
  interpolation in Table 1 of James [5]. For the option "love" the shape of
  the star is calculated using equation (10) and (11) from Correia [6] using
  the fluid second Love number for radial displacement, h_f (Correia equation
  (8)). The offset, D, is calculated using the approximation D=q.(R/d)^4,
  i.e., equation (38) from Chandrasekhar [4] with Delta_3=1 and nu = radius/d, 
  where d is the separation of the stars and q is the mass ratio.

    In eccentric orbits, the volume of the star is assumed to be constant. In
  general, the volume is calculated from the volume of the approximating
  ellipsoid. In the case of synchronous rotation, the volume of the star can
  be calculated using equation (2.18) from Kopal "Dynamics of Close Binary
  Systems" (Springer, 1978) by selecting the star shape model "roche_v".
   
   The simplified reflection model is approximately equivalent to Lambert
  law scattering with the coefficients heat_1 and heat_2  being equal to
  A_g/2, where A_g is the geometric albedo.

  Example
  -------
  >>> import ellc
  >>> import numpy as np
  >>> import matplotlib.pyplot as plt
  >>> t = np.arange(-0.25,0.75, 0.001)
  >>> spots_1 = [[30,180],[45,-45],[25,35],[0.2,0.8]]
  >>> flux = ellc.lc(t,radius_1=0.1,radius_2=0.05,sbratio=0.2,
  ...   incl=89.95,q=0.5,ld_1='quad',ldc_1=[0.65,0.2],ld_2='lin',ldc_2=0.45,
  ...   shape_1='poly3p0',shape_2='poly1p5',spots_1=spots_1)
  >>> plt.plot(t,flux)
  >>> plt.show()

  References
  ----------
  .. [1] Maxted, P.F.L. 2016. A fast, flexible light curve model for detached
      eclipsing binary stars and transiting exoplanets. A&A 591, A111, 2016.
  .. [2] Eker, 1994, ApJ, 420, 373.
  .. [3] Wilson, 1979, ApJ, 234, 1054.
  .. [4] Chandrasekhar, 1933, MNRAS, 93, 449.
  .. [5] James, 1964, ApJ, 140, 552.
  .. [6] Correia, 2014, A&A 570, L5.

  """


  # Copy control parameters into an np.array

  gridname_to_gridsize = {
    "very_sparse" :  4,
    "sparse"       :  8,
    "default"      : 16,
    "fine"         : 24,
    "very_fine"   : 32,
  }
  n1 = gridname_to_gridsize.get(grid_1,None)
  if n1 is None:
    raise Exception("Invalid grid size name")
  n2 = gridname_to_gridsize.get(grid_2,None)
  if n2 is None:
    raise Exception("Invalid grid size name")

  ldstr_to_ldcode = {
    "none"   :  0,
    "lin"    :  1,
    "quad"   :  2,
    "sing"   :  3,
    "claret" :  4,
    "log"    :  5,
    "sqrt"   :  6,
    "exp"    :  7,
    "power-2":  8,
    "mugrid" : -1
  }
  if ld_1 is None:
    ldstr_1 = 'none'
  else:
    ldstr_1 = ld_1

  if ld_2 is None:
    ldstr_2 = 'none'
  else:
    ldstr_2 = ld_2

  l1 = ldstr_to_ldcode.get(ldstr_1,None)
  if l1 is None:
    raise Exception("Invalid limb darkening law name")

  l2 = ldstr_to_ldcode.get(ldstr_2,None)
  if l2 is None:
    raise Exception("Invalid limb darkening law name")

  shapename_to_shapecode = {
    "roche_v" : -2,
    "roche"   : -1,
    "sphere"  :  0,
    "poly1p5" :  1,
    "poly3p0" :  2,
    "love"    :  3
  }
  s1 = shapename_to_shapecode.get(shape_1,None)
  if s1 is None:
    raise Exception("Invalid star shape name")
  s2 = shapename_to_shapecode.get(shape_2,None)
  if s2 is None:
    raise Exception("Invalid star shape name")

  if spots_1 is None:
    spar_1 = np.zeros([1,1])
    n_spots_1 = 0
  else:
    spar_1 = np.array(spots_1)
    if (spar_1.ndim != 2) or (spar_1.shape[0] != 4 ):
      raise Exception("spots_1 is not  (4, n_spots_1) array_like")
    n_spots_1 = spar_1.shape[1]

  if spots_2 is None:
    spar_2 = np.zeros([1,1])
    n_spots_2 = 0
  else:
    spar_2 = np.array(spots_2)
    if (spar_2.ndim != 2) or (spar_2.shape[0] != 4 ):
      raise Exception("spots_2 is not  (4, n_spots_2) array_like")
    n_spots_2 = spar_2.shape[1]

  ipar = np.array([n1,n2,n_spots_1,n_spots_2,l1,l2,s1,s2,1,0+exact_grav],
      dtype=int)

  if ld_1 == 'mugrid':
      try:
          mugrid_1 = np.array(ldc_1)
          n_mugrid_1 = len(mugrid_1)
      except:
          raise Exception("failed to obtain limb darkening data from ldc_1")
      if (n_mugrid_1 < 2):
          raise Exception("ldc_1 must have at least 2 elements")
  else:
      mugrid_1 = np.array([0.])
      n_mugrid_1 = 0

  if ld_2 == 'mugrid':
      try:
          mugrid_2 = np.array(ldc_2)
          n_mugrid_2 = len(mugrid_2)
      except:
          raise Exception("failed to obtain limb darkening data from ldc_2")
      if (n_mugrid_2 < 2):
          raise Exception("ldc_2 must have at least 2 elements")
  else:
      mugrid_2 = np.array([0.])
      n_mugrid_2 = 0

  # Copy binary parameters into an np.array

  if (radius_1 <= 0) or (radius_1 > 1):
    raise ValueError("radius_1 argument out of range")
  if (radius_1 == 1) and (shape_1 != "roche"):
    raise ValueError("radius_1=1 only allowed for Roche potential")

  if (radius_2 <= 0) or (radius_2 > 1):
    raise ValueError("radius_2 argument out of range")
  if (radius_2 == 1) and (shape_2 != "roche"):
    raise ValueError("radius_2=1 only allowed for Roche potential")

  par = np.zeros(39)
  par[0] = t_zero
  par[1] = period
  par[2] = sbratio
  par[3] = radius_1
  par[4] = radius_2
  par[5] = incl
  par[6] = light_3

  if a is not None : par[7] = a

  if (f_c is None) and (f_s is None): 
    pass
  elif (f_c is not None) and (f_s is not None):
    par[8] = f_c
    par[9] = f_s
  else:
    raise Exception("Must specify both f_c and f_s or neither.")

  if q <= 0 :
    raise ValueError("Mass ratio q must be positive.")
  par[10] = q
  
  ld_to_n  = {
    "none"   :  0,
    "lin"    :  1,
    "quad"   :  2,
    "sing"   :  3,
    "claret" :  4,
    "log"    :  2,
    "sqrt"   :  2,
    "exp"    :  2,
    "power-2":  2,
    "mugrid" :  0 
  }
  ld_n_1 = ld_to_n.get(ldstr_1,None)
  try: 
      if (ld_n_1 > 0): 
          par[11:11+ld_n_1] = ldc_1
  except:
    raise Exception("ldc_1 and ld_1 are inconsistent")
  ld_n_2 = ld_to_n.get(ldstr_2,None)
  try: 
      if (ld_n_2 > 0): 
          par[15:15+ld_n_2] = ldc_2
  except:
    raise Exception("ldc_2 and ld_2 are inconsistent")

  if gdc_1 is not None : par[19] = gdc_1

  if gdc_2 is not None : par[20] = gdc_2

  if didt is not None : par[21] = didt

  if domdt is not None : par[22] = domdt

  par[23] = rotfac_1

  par[24] = rotfac_2

  if bfac_1 is not None : par[25] = bfac_1

  if bfac_2 is not None : par[26] = bfac_2

  if heat_1 is not None : 
    t = np.array(heat_1)
    if t.size == 1:
      par[27] = t
    elif t.size == 3:
      par[27:30] = t
    else:
      raise Exception('Invalid size for array heat_1')

  if heat_2 is not None : 
    t = np.array(heat_2)
    if t.size == 1:
      par[30] = t
    elif t.size == 3:
      par[30:33] = t
    else:
      raise Exception('Invalid size for array heat_2')

  if lambda_1 is not None : par[33] = lambda_1

  if lambda_2 is not None : par[34] = lambda_2

  if vsini_1 is not None : par[35] = vsini_1

  if vsini_2 is not None : par[36] = vsini_2

  if shape_1 == "love":
    if rotfac_1 != 1:
      raise Exception(
                'Non-synchronous rotation not implemented for shape_1="love"')
    if (hf_1 <= 2/3.) or (hf_1 > 5):
        raise Exception('Invalid value for hf_1')
    par[37] = hf_1

  if shape_2 == "love":
    if rotfac_2 != 1:
      raise Exception(
                'Non-synchronous rotation not implemented for shape_2="love"')
    if (hf_2 <= 2/3.) or (hf_2 > 5):
        raise Exception('Invalid value for hf_2')
    par[38] = hf_2
     
  t_obs_array = np.array(t_obs)
  n_obs = len(t_obs_array)
  if t_exp is None:
    t_exp_array = np.zeros(n_obs)
  else:
    t_exp_array = np.ones(n_obs)*t_exp

  if n_int is not None :
    if np.amax(n_int) < 1 : raise Exception("No n_int values > 1.") 
    if np.amin(n_int) < 0 : raise Exception("Invalid negative n_int value(s).") 
    n_int_array = np.array(np.ones(n_obs)*n_int, dtype=int)
  else:
    n_int_array = np.ones(n_obs, dtype=int)

  # Create list of times for calculation, weights for integration and
  # indices to relate these both back to the original t_obs array
  i_obs = np.arange(0,n_obs)
  t_calc = t_obs_array[n_int_array == 1]
  w_calc = np.ones_like(t_calc)
  i_calc = i_obs[n_int_array == 1]
  n_int_max = np.amax(n_int_array)
  for i_int in np.unique(n_int_array[n_int_array > 1]) :
    t_obs_i = t_obs_array[n_int_array == i_int]
    t_exp_i = t_exp_array[n_int_array == i_int]
    i_obs_i = i_obs[n_int_array == i_int]
    for j_int in range(0,i_int):
      t_calc = np.append(t_calc, t_obs_i+(j_int/(i_int-1.)-0.5)*t_exp_i)
      i_calc = np.append(i_calc,i_obs_i)
      if (j_int == 0) or (j_int == i_int-1):
        w_calc = np.append(w_calc, 0.5*np.ones_like(t_obs_i)/(i_int-1.))
      else:
        w_calc = np.append(w_calc, np.ones_like(t_obs_i)/(i_int-1.))

  lc_rv_flags = ellc_f.ellc.lc(t_calc,par,ipar,spar_1,spar_2,
                n_mugrid_1, mugrid_1,n_mugrid_2, mugrid_2, verbose)
  if ((np.sum(np.isnan(lc_rv_flags)) > 0 )) & (verbose > 0):
    lc_dummy = ellc_f.ellc.lc(t_calc,par,ipar,spar_1,spar_2,
               n_mugrid_1, mugrid_1,n_mugrid_2, mugrid_2,9)

  flux = np.zeros(n_obs)
  for j in range(0,len(t_calc)):
    flux[i_calc[j]] += lc_rv_flags[j,0]*w_calc[j]

  t_obs_0 = t_obs_array[n_int_array == 0 ] # Points to be interpolated
  n_obs_0 = len(t_obs_0)
  if n_obs_0 > 0 :
    i_sort = np.argsort(t_calc)
    t_int = t_calc[i_sort]
    f_int = lc_rv_flags[i_sort,0]
    flux[n_int_array == 0 ] = np.interp(t_obs_0,t_int,f_int)

  return flux
 

